#SkIndigo Ruby code is copyright Dale Martens 2011, all rights reserved, and
#Glare Technologies Limited, 2014 - 
#-----------------------------------------------------------------------------
	
class IndigoMaterial

	include SkIndigoUVS

	#These class variables are used as the material defaults
	@@dict="indigo07_materials"  #do not change this dictionary (needed for backward compatibility)
	@@type = "Phong"
	@@ior = 1.3
	@@cau=0.0
	@@fres=1.0
	@@sigma=0.5
	@@power=100
	@@ole=17.5
	@@exp=1000
	@@precedence=10
	@@temp=5000
	#@@g_spectrum=1.0
	@@g_spectrum=IndigoTexture.new.default_g_spectrum.to_s
	@@no_smooth="No"
	@@ies="No"
	@@ies_path=""
	@@sss="No"
	#@@sssval=0.0
	#@@sssval=IndigoTexture.new.default_sss_val.to_s
	@@phase_function="Uniform"
	@@albedo_texture=IndigoTexture.new.default_albedo.to_s
	@@blend_map=IndigoTexture.new.default_blend.to_s
	#@@bump_map="None|Default|0.0|0.0|0.0|1.0"
	@@bump_map=IndigoTexture.new.default_bump.to_s
	@@normal_map=IndigoTexture.new.default_normal.to_s
	@@exp_map=IndigoTexture.new.default_exponent.to_s
	@@clip_map=IndigoTexture.new.default_clip.to_s
	@@disp_map=IndigoTexture.new.default_displacement.to_s
	@@emission_map=IndigoTexture.new.default_emission.to_s
	@@sigma_map=IndigoTexture.new.default_sigma.to_s
	@@roughness_map=IndigoTexture.new.default_roughness.to_s
	@@fresnel_map=IndigoTexture.new.default_fresnel.to_s
	@@thickness_map=IndigoTexture.new.default_thickness.to_s
	@@specular_map=IndigoTexture.new.default_specular.to_s
	@@abs_layer_trans=IndigoTexture.new.default_abs_layer_trans.to_s
	@@absorption_map=IndigoTexture.new.default_absorption.to_s
	@@sss_map=IndigoTexture.new.default_sss_map.to_s
	@@g_spectrum_map=IndigoTexture.new.default_g_spectrum.to_s
	#@@base_emission=[255,255,255]
	@@base_emission=IndigoTexture.new.default_base_emission.to_s
	@@base_emission_gain=0.0
	@@base_emission_spectrum="Blackbody"
	@@layer=-1
	@@thumbnail=""
	@@view_dependent=true
	@@max_subdiv=10
	@@pixel_threshold=4.0
	@@curvature_threshold=0.1
	@@sub_smooth=false
	@@linked_igm=""
	@@use_bump=false
	@@use_normal=false
	@@use_displace=false
	@@use_emission=false
	@@use_abs_layer_trans=false
	@@use_clip=false
	@@use_specular=true
	@@step_blend=false
	@@matA="None"
	@@matB="None"
	@@blendfactor=0.5
	@@use_nkdata="No"
	@@emission_scale=1750.0
	@@emission_measure="luminous_flux"
	@@substrate="None"
	@@front_material="None"
	@@back_material="None"
	@@interference=false
	@@arch_glass=false
	@@single_face=false
	@@emission_sampling_factor=1.0
	@@backface_emit = false
	@@mask_name = ""

	@@defaults={'front_material_name'=>'None','back_material_name'=>'None'}
	@@transmittance_map=IndigoTexture.new.default_transmittance.to_s
	@@front_roughness=IndigoTexture.new.default_roughness.to_s
	@@back_roughness=IndigoTexture.new.default_roughness.to_s
	@@front_fresnel_scale=IndigoTexture.new.default_fresnel.to_s
	@@back_fresnel_scale=IndigoTexture.new.default_fresnel.to_s
	@@r_f=IndigoTexture.new.default_r_f.to_s


	attr_reader :mat

	def initialize(su_material)
		@mat=su_material
		
		#if self.type == "Thin Glass"
		#	puts "Converting material from thin glass to specular"
		#	self.type = "Specular"
		#	self.arch_glass = true
		#end
				
		#following code is for backward compatibility
		dicts=@mat.attribute_dictionaries if @mat
		return if not dicts
		if dicts["indigo_blend"]
			dict=dicts["indigo_blend"]
			self.blend_factor=dict["blendfactor"]
			self.a_name=dict["matA"]
			self.b_name=dict["matB"]
			dicts.delete(dict)
		end
		if dicts["indigo07_materials"]
			dict=dicts["indigo07_materials"]
			keys=dict.keys
			if keys.index("bump_value")
				b=dict["bump_value"]
				b=0.0 if not b
				dict.delete_key("bump_value")
				bm=self.bump_map
				bm.b=b
				self.bump_map=bm
			end
			if keys.index("expmap_value")
				b=dict["expmap_value"]
				b=1000 if not b
				dict.delete_key("expmap_value")
				em=self.exponent_map
				em.b=b
				self.exponent_map=em
			end
			if keys.index("exp")
				exp=dict["exp"]
				exp=1000 if not exp
				dict.delete_key("exp")
				em=self.exponent_map
				em.b=exp
				self.exponent_map=em
			end
			if keys.index("temp")  #apply temperature attribute to the base emission black body temperature
				temp=dict["temp"]
				temp=5000 if not temp
				dict.delete_key("temp")
				be=self.base_emission
				be.bb_temperature=temp
				self.base_emission=be
			end	
			if keys.index("g_spectrum") #new implementation of g spectrum in SkIndigo 2.4
				gs=dict["g_spectrum"]
				gs=1.0 if not gs
				gs=gs.to_s.split(' ')
				g_spectrum_map=self.g_spectrum_map
				if gs.length==3 #rgb spectrum
					g_spectrum_map.rgb_r=gs[0].to_f
					g_spectrum_map.rgb_g=gs[1].to_f
					g_spectrum_map.rgb_b=gs[2].to_f
					g_spectrum_map.spectrum_type="RGB"
				else
					g_spectrum_map.uni_value=gs[0].to_f
					g_spectrum_map.type="Uniform"
				end
				self.g_spectrum_map=g_spectrum_map
				dict.delete_key("g_spectrum")
			end
			if keys.index("sssval")  #new implementation of sss in SkIndigo 2.4
				sss=dict["sssval"]
				sss=0.0 if not sss
				sss=sss.to_s.split(' ')
				sss_map=self.sss_map
				if sss.length==3 #rgb spectrum
					sss_map.rgb_r=sss[0].to_f
					sss_map.rgb_g=sss[1].to_f
					sss_map.rgb_b=sss[2].to_f
					sss_map.spectrum_type="RGB"
				else
					sss_map.uni_value=sss[0].to_f
					sss_map.type="Uniform"
				end
				self.sss_map=sss_map
				dict.delete_key("sssval")
			end
			if keys.index("base_emission")
				be=dict["base_emission"]
				be=[0.0,0.0,0.0] if not be
				new_be=self.base_emission
				new_be.rgb_r=be[0]
				new_be.rgb_g=be[1]
				new_be.rgb_b=be[2]
				self.base_emission=new_be
				dict.delete_key("base_emission")
			end
			if keys.index("base_emission_spectrum")
				bes=dict["base_emission_spectrum"]
				bes="Blackbody" if not bes
				be=self.base_emission
				if bes=="Blackbody"
					be.spectrum_type="Blackbody"
				else
					be.spectrum_type="RGB"
				end
				self.base_emission=be
				dict.delete_key("base_emission_spectrum")
			end
			if keys.index("power")
				power=dict["power"]
				power=100.0 if not power
				ole=dict["ole"]  #the overall luminous efficacy
				ole=17.5 if not ole
				self.emission_scale=power.to_f*ole.to_f
				self.emission_measure="luminous_flux"
				dict.delete_key("power")
			end
			
			
			#following code is due to deprecation of blackbody and rgb lights - now are converted to light-emitting materials
			t=self.type
			if (t=="Blackbody Light") or (t=="RGB Light")
				be=self.base_emission
				be.spectrum_type="Blackbody" if t =="Blackbody Light"
				be.spectrum_type="RGB" if t=="RGB Light"
				self.base_emission=be
				self.type="Diffuse"  #change the type to diffuse
				self.layer=0         #set the layer to 0 to ensure the material will emit light when rendering
			end
		end
		
		
		
		if dicts["uvsets"]
			dict=dicts["uvsets"]
			keys=dict.keys
			if keys.index("albedo")
				set=dict["albedo"]
				set="Default" if not set
				dict.delete_key("albedo")
				am=self.albedo_texture
				am.uv_set=set
				self.albedo_texture=am
			end
			if keys.index("bump")
				set=dict["bump"]
				set="Default" if not set
				dict.delete_key("bump")
				bm=self.bump_map
				bm.uv_set=set
				self.bump_map=bm
			end
			if keys.index("exponent")
				set=dict["exponent"]
				set="Default" if not set
				dict.delete_key("exponent")
				em=self.exponent_map
				em.uv_set=set
				self.exponent_map=em
			end
			if keys.index("blend")
				set=dict["blend"]
				set="Default" if not set
				dict.delete_key("blend")
				bm=self.blend_map
				bm.uv_set=set
				self.blend_map=bm
			end
			if keys.index("clip")
				set=dict["clip"]
				set="Default" if not set
				dict.delete_key("clip")
				cm=self.clip_map
				cm.uv_set=set
				self.clip_map=cm
			end
			dicts.delete(dict)
		end
	end
	
	
	# Determine a good Indigo material type based on the SketchUp material properties
	def compute_auto_converted_type(sketchup_mat)
		
		if mat.alpha < 1.0
			return "Specular"
		else
			# Background material should be exported as diffuse.
			return "Diffuse" if (sketchup_mat.get_attribute(@@dict, 'layer', -1) != -1) # Return as diffuse if !emitter.
				
			if name =~ /metal/i || name =~ /chrome/i || name =~ /alumin/i || name =~ /steel/i
				return "Metals"
			else
				return "Phong"
			end
		end
	end
	

		
		
	def reset() #deletes all indigo attributes

		IndigoMaterial.reset_indigo_attributes(@mat)
	end


	# We'll make this a class method so it can operate on SketchUp materials without having to create an Indigo material first.
	def IndigoMaterial.reset_indigo_attributes(skup_mat)
		
		dicts = skup_mat.attribute_dictionaries
		if dicts != nil
			dicts.delete(@@dict)
		end
	end


	def IndigoMaterial.print_out_attributes(skup_mat)
		puts "attributes for sketchup material #{skup_mat}..."
		puts "skup_mat.attribute_dictionaries: #{skup_mat.attribute_dictionaries.inspect}"
		if !skup_mat.attribute_dictionaries.nil?
			skup_mat.attribute_dictionaries.each do |dict|
				dict.each_pair do |key, value|
					puts [dict,key,value]
				end
			end
		end
	end

	###delete this material - code used from TIG's material-delete ruby snippet
	def delete!

		model=Sketchup.active_model
		ents=model.entities; mats=model.materials
		group=ents.add_group();gents=group.entities ### temporarily use other materials
		temp=gents.add_group()
		(mats.to_a-[@mat]).each{|m|tc=temp.copy;tc.material=m}
		mats.current=nil if mats.current==@mat ### ensure self is not current material
		mats.purge_unused ### purge self from browser
		group.erase! ### erase! the temporary material user, use set as was.
	end

	
	
	###copy parameters from this Indigo material to another
	def copy(im2)

		im2.reset()
		mat1=self.mat
		mat2=im2.mat  #get the su material of the 
		dicts=mat1.attribute_dictionaries
		dicts.each {|dict|
			dict.each_pair {|key,value|
				mat2.set_attribute(dict.name,key,value)
				#p [dict,key,value]
			}
		}
		mat2.color=mat1.color
		if mat1.texture!=nil
			tex_path=mat1.texture.filename
			if FileTest.exist?(tex_path)
				mat2.texture=mat1.texture.filename
			else
				temp=SkIndigo.temp_dir
				tex_path=File.join(temp,mat1.texture.filename)
				tex_path=save_mat_texture(mat1,tex_path)
				if tex_path!=nil and FileTest.exist?(tex_path)
					mat2.texture=tex_path
				end
			end
		end

		mat2.alpha=mat1.alpha
	end

	####################Method to find a SU material based on name
	#note: prior to Indigo 2.0.X, this method was based on display_name, not name.
	#this was changed to eliminate duplicate material naming conflicts
	def IndigoMaterial.find(name) 
		materials=Sketchup.active_model.materials
		mat=materials[name]
		if mat
			return IndigoMaterial.new(mat)
		else
			return nil
		end
	end

	#####################################returns and existing material or creates a new material with the given texture image
	#method was used for internal texture support of previous skindigo versions - not used currently
	def IndigoMaterial.find_texture_file(path)

		filename=File.basename(path)
		mats=Sketchup.active_model.materials
		textures=mats.collect {|m| (m.texture ? File.basename(m.texture.filename) : nil)}
		names=mats.collect {|m| m.display_name}
		index=textures.index(filename)
		if index
			return IndigoMaterial.new(mats[index])
		else
			new_mat=mats.add(File.basename(filename,".*"))
			if FileTest.exist?(path)
				new_mat.texture=path
				UI.messagebox "New texture material created name #{new_mat.display_name}"
				return IndigoMaterial.new(new_mat)
			else
				UI.messagebox "Could not find or create material with texture #{filename}"
				return nil
			end
		end
	end

	#optimized method to check or emitters without creating an IndigoMaterial object
	def IndigoMaterial.emitter?(su_mat)

		return false if su_mat==nil
		layer=su_mat.get_attribute(@@dict,'layer',-1)
		type=su_mat.get_attribute(@@dict,'type',"Diffuse") 
		return false if type=="Blend"
		if layer!=(-1)
			return true
		else
			return false
		end
	end

	#####new attribute setting and retrievel methods test
	def get(attr,texture=nil)

		val=mat.get_attribute(@@dict,attr,@@defaults[attr])
		if texture!=nil
			return IndigoTexture.new(val)
		else
			return val
		end
	end

	##
	def set(attr,val,texture=nil)

		if texture!=nil
			mat.set_attribute(@@dict,attr,val.to_s)
		else
			mat.set_attribute(@@dict,attr,val)
		end
	end

	def name
		return @mat.name
	end

	def rgb
		return mat.color.red,mat.color.green,mat.color.blue
	end
	
	
	########################Attribute read and writer functions########################33
	def type
		t = mat.get_attribute(@@dict, 'type')  # Check if the type has been set
		if not t
			return compute_auto_converted_type(mat)
		else
			return t
		end
	end

	def indigo_type
		t=self.type
		case t
		when "Diffuse"
			if mat.alpha==0 
				xml="diffuse_transmitter"
			else
				xml="diffuse"
			end
		when "Specular","Phong","Blend","Coating"
			xml=t.downcase
		when "Metals"
			xml="phong"
		when "Glossy Transparent"
			xml="glossy_transparent"
		when "Oren-Nayar"
			xml="oren_nayar"
		when "Double-Sided Thin"
			xml="double_sided_thin"
		when "Simple Thin"
			xml="double_sided_thin"
		when "Exit Portal"
			xml="exit_portal\/"
		when "Fast SSS"
			xml="fast_sss"
		else
			xml=t
		end
	end

	def texture
		mat.texture
	end

	def texture=(value)
		mat.texture=(value)
	end

	def thumbnail
		mat.get_attribute(@@dict,'thumbnail',@@thumbnail)
	end

	def thumbnail=(path)
		mat.set_attribute(@@dict,'thumbnail',path)
	end

	def type=(value)
		mat.set_attribute(@@dict,'type',value)
	end

	def ior
		mat.get_attribute(@@dict,'ior',@@ior)
	end

	def ior=(value)
		mat.set_attribute(@@dict,'ior',value)
	end

	def fresnel_scale
		mat.get_attribute(@@dict,'fres',@@fres)
	end

	def fresnel_scale=(value)
		mat.set_attribute(@@dict,'fres',value)
	end

	def sigma
		mat.get_attribute(@@dict,'sigma',@@sigma)	
	end

	def sigma=(value)
		mat.set_attribute(@@dict,'sigma',value)
	end

	def cauchy_b_coeff
		mat.get_attribute(@@dict,'cau',@@cau)
	end

	def cauchy_b_coeff=(value)
		mat.set_attribute(@@dict,'cau',value)
	end

	def power_drawn
		mat.get_attribute(@@dict,'power',@@power)
	end

	def power_drawn=(value)
		mat.set_attribute(@@dict,'power',value)
	end

	def overall_luminous_efficacy
		mat.get_attribute(@@dict,'ole',@@ole)
	end

	def overall_luminous_efficacy=(value)
		mat.set_attribute(@@dict,'ole',value)
	end
		
	def exponent
		mat.get_attribute(@@dict,'exp',@@exp)
	end

	def exponent=(value)
		#mat.set_attribute(@@dict,'exp',value)
		em=self.exponent_map
		em.b=value
		self.exponent_map=em
	end

	def precedence
		mat.get_attribute(@@dict,'precedence',@@precedence)
	end

	def precedence=(value)
		mat.set_attribute(@@dict,'precedence',value)
	end

	def temperature
		mat.get_attribute(@@dict,'temp',@@temp)
	end

	def temperature=(value)
		mat.set_attribute(@@dict,'temp',value)
	end

	def g_spectrum
		mat.get_attribute(@@dict,'g_spectrum',@@g_spectrum)
	end

	def g_spectrum=(value)
		mat.set_attribute(@@dict,'g_spectrum',value)
	end

	def smooth?  #tas of version 1.1.1 this method returns whether subdivision smoothing is enabled (no longer used for normal smoothing)
		mat.get_attribute(@@dict,'sub_smooth',@@sub_smooth)
	end

	def smooth=(value)
		if value
			mat.set_attribute(@@dict,'sub_smooth',true)  #enables subdivision smoothing
		else
			mat.set_attribute(@@dict,'sub_smooth',false)  #disables subdivision smoothing
		end
	end

	def ies?
		use_ies=mat.get_attribute(@@dict,'ies',@@ies)
		return true if use_ies=="Yes"
		return false if use_ies=="No"
	end

	def ies=(value)
		if value
			mat.set_attribute(@@dict,'ies',"Yes")  #enable ies profile
		else
			mat.set_attribute(@@dict,'ies',"No")  #disables ies profile
		end
	end

	def use_nkdata?
		use_nkdata=mat.get_attribute(@@dict,'use_nkdata',@@use_nkdata)
		return true if use_nkdata=="Yes"
		return false if use_nkdata=="No"
	end

	def use_nkdata=(value)
		if value
			mat.set_attribute(@@dict,'use_nkdata',"Yes")
		else
			mat.set_attribute(@@dict,'use_nkdata',"No")
		end
	end

	def ies_profile
		path=mat.get_attribute(@@dict,'ies_path',@@ies_path)
		return IESProfile.new(path)
	end

	def ies_profile=(value)  #value is an IESProfile object
		mat.set_attribute(@@dict,'ies_path',value.path)
	end

	def linked_igm
		path=mat.get_attribute(@@dict,'linked_igm',@@linked_igm)
		return path
	end

	def linked_igm=(value)
		value=value.gsub(/\\/,"\/")
		mat.set_attribute(@@dict,'linked_igm',value)
	end

	def valid_igm_reference?
		path=self.linked_igm.to_s
		if ((FileTest.exist?(path)) or (path==""))
			return true
		else
			return false
		end
	end

	def absorption_spectrum=(value)

		#p value
		rgb_ab=value.split(" ")
		#p rgb_ab
		if rgb_ab.length==1  #uniform
			#todo
		elsif rgb_ab.length==3 #rgb
			red_ab=rgb_ab[0].to_f
			green_ab=rgb_ab[1].to_f
			blue_ab=rgb_ab[2].to_f
			opacity=(Math.sqrt([red_ab,green_ab,blue_ab].max*10)).round
			opacity=99 if opacity>99
			opacity=1 if opacity<1
			red=((1-(10*red_ab)/(opacity**2))*255).round
			red=0 if red<0
			red=255 if red>255
			blue=((1-(10*blue_ab)/(opacity**2))*255).round
			blue=0 if blue<0
			blue=255 if blue>255
			green=((1-(10*green_ab)/(opacity**2))*255).round
			green=0 if green<0
			green=255 if green>255
			set_alpha=opacity.to_f/100
			self.alpha=set_alpha
			self.color=[red,green,blue]
		end
	end

	def absorption_spectrum

		opacity=mat.alpha
		alpha_factor=opacity*opacity*1000
		rgb=self.rgb
		absorption="%.4f" %((1.0-rgb[0]/255.0)*alpha_factor) + " "+ "%.4f" %((1.0-rgb[1]/255.0)*alpha_factor) + " "+ "%.4f" %((1.0-rgb[2]/255.0)*alpha_factor)
		#absorption=((1.0-rgb[0]/255.0)*alpha_factor),((1.0-rgb[1]/255.0)*alpha_factor),((1.0-rgb[2]/255.0)*alpha_factor)
		return absorption.split(" ")
	end

	def absorption_color

		rgb=self.rgb
		return [255.0-rgb[0],255.0-rgb[1],255.0-rgb[2]]
	end

	def sss?
		use_sss=mat.get_attribute(@@dict,'sss',@@sss)
		return true if use_sss=="Yes"
		return false if use_sss=="No"
	end

	def sss=(value)
		if value
			mat.set_attribute(@@dict,'sss',"Yes")  #enable sub-surface scattering
		else
			mat.set_attribute(@@dict,'sss',"No")  #disables sub-surface scattering
		end
	end

	def sss_spectrum
		mat.get_attribute(@@dict,'sssval',@@sssval)
	end

	def sss_spectrum=(value)
		#TODO validation
		mat.set_attribute(@@dict,'sssval',value)
	end	

	def phase_function
		mat.get_attribute(@@dict,'phase_function',@@phase_function)
	end

	def phase_function=(value)
		mat.set_attribute(@@dict,'phase_function',value)
	end

	def albedo_texture?
		t=self.type
		case t
		when "Specular","Glossy Transparent","Blend","Thin Glass","RGB Light","Blackbody Light","Exit Portal","Linked IGM","Metals"
			return false
		else
			return texture?(self.albedo_texture)
		end
				
	end

	def albedo_shader?
		t=self.type
		case t
		when "Specular","Glossy Transparent","Blend","Thin Glass","RGB Light","Blackbody Light","Exit Portal","Linked IGM","Metals"
			return false
		else
			return self.albedo_texture.procedural?
		end
	end


	def albedo_texture
		tex=IndigoTexture.new(mat.get_attribute(@@dict,'albedo_map',@@albedo_texture))
		return tex
	end

	def albedo_texture=(tex)  
		mat.set_attribute(@@dict,'albedo_map',tex.to_s)	
	end


	def abs_layer_trans_map?
		return false if !use_abs_layer_trans?
		t=self.type
		case t
		when "Specular","Glossy Transparent"
			return texture?(self.abs_layer_trans_map)
		else
			return false
		end
				
	end

	def abs_layer_trans_shader?
		t=self.type
		case t
		when "Specular","Glossy Transparent"
			return self.abs_layer_trans_map.procedural?
		else
			return false
		end
	end


	def abs_layer_trans_map
		tex=IndigoTexture.new(mat.get_attribute(@@dict,'abs_layer_trans',@@abs_layer_trans))
		return tex
	end

	def abs_layer_trans_map=(tex)  
		mat.set_attribute(@@dict,'abs_layer_trans',tex.to_s)	
	end

	def use_abs_layer_trans?
		value=mat.get_attribute(@@dict,'use_abs_layer_trans',@@use_abs_layer_trans)
		value=false unless self.abs_layer_trans_map.valid?
		return value
	end

	def use_abs_layer_trans=(value)
		mat.set_attribute(@@dict,'use_abs_layer_trans',value)
	end

	def specular_map?
		return false if !use_specular?
		t=self.type
		case t
		when "Metals"
			return texture?(self.specular_map)
		else
			return false
		end
				
	end

	def specular_shader?
		t=self.type
		case t
		when "Metals"
			return self.specular_map.procedural?
		else
			return false
		end
	end


	def specular_map
		tex=IndigoTexture.new(mat.get_attribute(@@dict,'specular_map',@@specular_map))
		return tex
	end

	def specular_map=(tex)  
		mat.set_attribute(@@dict,'specular_map',tex.to_s)	
	end

	def use_specular?
		value=mat.get_attribute(@@dict,'use_specular',@@use_specular)
		value=false unless self.specular_map.valid?
		return value
	end

	def use_specular=(value)
		mat.set_attribute(@@dict,'use_specular',value)
	end

	def blend_map?
		t=self.type
		case t
		when "Blend"
			return texture?(self.blend_map)
		else
			return false
		end
	end

	def blend_shader?
		t=self.type
		case t
		when "Blend"
			return self.blend_map.procedural?
		else
			return false		
		end
	end

	def blend_map
		tex=IndigoTexture.new(mat.get_attribute(@@dict,'blend_map',@@blend_map))
		return tex
	end

	def blend_map=(tex)  
		mat.set_attribute(@@dict,'blend_map',tex.to_s)	
	end

	def step_blend?
		mat.get_attribute(@@dict,'step_blend',@@step_blend)
	end

	def step_blend=(value)
		mat.set_attribute(@@dict,'step_blend',value)
	end

	def interference?
		mat.get_attribute(@@dict,'interference',@@interference)
	end

	def interference=(value)
		mat.set_attribute(@@dict,'interference',value)
	end
	
	def arch_glass?
		mat.get_attribute(@@dict,'arch_glass',@@arch_glass)
	end

	def arch_glass=(value)
		mat.set_attribute(@@dict,'arch_glass',value)
	end
	
	def single_face?
		mat.get_attribute(@@dict,'single_face',@@single_face)
	end

	def single_face=(value)
		mat.set_attribute(@@dict,'single_face',value)
	end

	def bump_map
		tex=mat.get_attribute(@@dict,'bump_map',@@bump_map)
		return IndigoTexture.new(tex)
	end

	def bump_map=(tex)  #tex is an IndigoTexture object
		mat.set_attribute(@@dict,'bump_map',tex.to_s)	
	end

	def bump_map?
		return false if !use_bump?
		t=self.type
		case t
		when "Blend","RGB Light","Blackbody Light","Exit Portal","Linked IGM"
			return false
		else
			#return false if self.indigo_type=="diffuse_transmitter"
			return texture?(self.bump_map)
		end
	end

	def bump_shader?
		t=self.type
		case t
		when "Blend","RGB Light","Blackbody Light","Exit Portal","Linked IGM"
			return false
		else
			#return false if self.indigo_type=="diffuse_transmitter"
			return self.bump_map.procedural?
		end
	end

	def use_bump?
		value=mat.get_attribute(@@dict,'use_bump',@@use_bump)
		value=false unless self.bump_map.valid?
		value=false if self.type=="Blend"
		return value
	end

	def use_bump=(value)
		mat.set_attribute(@@dict,'use_bump',value)
	end

	def normal_map
		tex=mat.get_attribute(@@dict,'normal_map',@@normal_map)
		return IndigoTexture.new(tex)
	end

	def normal_map=(tex)  #tex is an IndigoTexture object
		mat.set_attribute(@@dict,'normal_map',tex.to_s)	
	end

	def normal_map?
		return false if !use_normal?
		t=self.type
		case t
		when "Blend","RGB Light","Blackbody Light","Exit Portal","Linked IGM"
			return false
		else
			#return false if self.indigo_type=="diffuse_transmitter"
			return texture?(self.normal_map)
		end
	end

	def normal_shader?
		t=self.type
		case t
		when "Blend","RGB Light","Blackbody Light","Exit Portal","Linked IGM"
			return false
		else
			#return false if self.indigo_type=="diffuse_transmitter"
			return self.normal_map.procedural?
		end
	end

	def use_normal?
		value=mat.get_attribute(@@dict,'use_normal',@@use_normal)
		value=false unless self.normal_map.valid?
		value=false if self.type=="Blend"
		return value
	end

	def use_normal=(value)
		mat.set_attribute(@@dict,'use_normal',value)
	end

	def disp_map
		tex=mat.get_attribute(@@dict,'disp_map',@@disp_map)
		return IndigoTexture.new(tex)
	end

	def disp_map=(tex)
		mat.set_attribute(@@dict,'disp_map',tex.to_s)
	end

	def disp_map?
		return false if !use_displace?
		t=self.type
		case t
		when "RGB Light","Blackbody Light","Exit Portal","Linked IGM", "Blend"
			return false
		else
			return texture?(self.disp_map)
		end
	end

	def disp_shader?
		t=self.type
		case t
		when "RGB Light","Blackbody Light","Exit Portal","Linked IGM"
			return false
		else
			return self.disp_map.procedural?
		end

	end

	def use_displace?
		value=mat.get_attribute(@@dict,'use_displace',@@use_displace)
		value=false unless self.disp_map.valid?
		value=false if self.type=="Blend"
		return value
	end

	def use_displace=(value)
		mat.set_attribute(@@dict,'use_displace',value)
	end



	def emitter?
		t=self.type
		case t
		when "Blend"
			return false
		else
			#gain=self.base_emission_gain
			#if gain.to_f>0.0 and self.layer!="-1"
			if self.layer!=(-1)
				return true
			else
				return false
			end
		end
	end	

	def base_emission
		tex=mat.get_attribute(@@dict,'base_emission2',@@base_emission)
		tex=IndigoTexture.new(tex)
		return tex
	end

	def base_emission=(tex)
		mat.set_attribute(@@dict,'base_emission2',tex.to_s)
	end
	

	def emission_map
		tex=mat.get_attribute(@@dict,'emission_map',@@emission_map)
		return IndigoTexture.new(tex)
	end

	def emission_map=(tex)
		mat.set_attribute(@@dict,'emission_map',tex.to_s)
	end

	def emission_map?
		return false if !use_emission?
		t=self.type
		case t
		when "Blend","RGB Light","Blackbody Light","Exit Portal","Linked IGM"
			return false
		else
			return texture?(self.emission_map)
		end
	end

	def emission_shader?
		t=self.type
		case t
		when "Blend","RGB Light","Blackbody Light","Exit Portal","Linked IGM"
			return false
		else
			return self.emission_map.procedural?
		end

	end

	def use_emission?
		value=mat.get_attribute(@@dict,'use_emission',@@use_emission)
		value=false unless self.emission_map.valid?
		value=false if self.type=="Blend"
		return value
	end

	def use_emission=(value)
		mat.set_attribute(@@dict,'use_emission',value)
	end

	####
	def transmittance_map
		tex=mat.get_attribute(@@dict,'transmittance_map',@@transmittance_map)
		return IndigoTexture.new(tex)
	end

	def transmittance_map=(tex)
		mat.set_attribute(@@dict,'transmittance_map',tex.to_s)
	end

	def transmittance_map?
		t=self.type
		case t
		when "Double-Sided Thin", "Simple Thin"
			return texture?(self.transmittance_map)
		else
			false
		end
	end

	def transmittance_shader?
		t=self.type
		case t
		when "Double-Sided Thin", "Simple Thin"
			return self.transmittance_map.procedural?
		else
			false
		end

	end

	####
	def absorption_map
		tex=mat.get_attribute(@@dict,'absorption_map',@@absorption_map)
		return IndigoTexture.new(tex)
	end

	def absorption_map=(tex)
		mat.set_attribute(@@dict,'absorption_map',tex.to_s)
	end

	####
	def sss_map
		tex=mat.get_attribute(@@dict,'sss_map',@@sss_map)
		return IndigoTexture.new(tex)
	end

	def sss_map=(tex)
		mat.set_attribute(@@dict,'sss_map',tex.to_s)
	end

	####
	def g_spectrum_map
		tex=mat.get_attribute(@@dict,'g_spectrum_map',@@g_spectrum_map)
		return IndigoTexture.new(tex)
	end

	def g_spectrum_map=(tex)
		mat.set_attribute(@@dict,'g_spectrum_map',tex.to_s)
	end
	######
	def sigma_map
		tex=mat.get_attribute(@@dict,'sigma_map',@@sigma_map)
		return IndigoTexture.new(tex)
	end

	def sigma_map=(tex)
		mat.set_attribute(@@dict,'sigma_map',tex.to_s)
	end

	def sigma_map?
		t=self.type
		case t
		when "Diffuse","Specular","Phong","Glossy Transparent","Blend","Metals","Thin Glass","RGB Light","Blackbody Light","Exit Portal","Linked IGM"
			return false
		else
			return texture?(self.sigma_map)
		end
	end

	def sigma_shader?

		t=self.type
		case t
		when "Diffuse","Specular","Phong","Glossy Transparent","Blend","Metals","Thin Glass","RGB Light","Blackbody Light","Exit Portal","Linked IGM"
			return false
		else
			return self.sigma_map.procedural?
		end
	end
	
	######
	def is_roughness_param_stored?
		# Try and read roughness param with nil default, and return true if result was not nil.
		return !mat.get_attribute(@@dict,'roughness_map', nil).nil?
	end
	
	def roughness_map
		tex = mat.get_attribute(@@dict,'roughness_map',@@roughness_map)
		return IndigoTexture.new(tex)
	end

	def roughness_map=(tex)
		mat.set_attribute(@@dict,'roughness_map',tex.to_s)
	end

	def roughness_map?
		t=self.type
		case t
		when "Coating", "Phong", "Glossy Transparent", "Metals", "Fast SSS"
			return texture?(self.roughness_map)
		else
			return false
		end
	end

	def roughness_shader?

		t=self.type
		case t
		when "Coating"
			return self.roughness_map.procedural?
		else
			return false
		end
	end

	######
	def front_roughness_map
		tex=mat.get_attribute(@@dict,'front_roughness_map',@@front_roughness)
		return IndigoTexture.new(tex)
	end

	def front_roughness_map=(tex)
		mat.set_attribute(@@dict,'front_roughness_map',tex.to_s)
	end

	def front_roughness_map?
		t=self.type
		case t
		when "Double-Sided Thin", "Simple Thin"
			return texture?(self.front_roughness_map)
		else
			return false
		end
	end

	def front_roughness_shader?

		t=self.type
		case t
		when "Double-Sided Thin", "Simple Thin"
			return self.front_roughness_map.procedural?
		else
			return false
		end
	end

	######
	def back_roughness_map
		tex=mat.get_attribute(@@dict,'back_roughness_map',@@back_roughness)
		return IndigoTexture.new(tex)
	end

	def back_roughness_map=(tex)
		mat.set_attribute(@@dict,'back_roughness_map',tex.to_s)
	end

	def back_roughness_map?
		t=self.type
		case t
		when "Double-Sided Thin", "Simple Thin"
			return texture?(self.back_roughness_map)
		else
			return false
		end
	end

	def back_roughness_shader?

		t=self.type
		case t
		when "Double-Sided Thin", "Simple Thin"
			return self.back_roughness_map.procedural?
		else
			return false
		end
	end

	######
	def front_fresnel_scale_map
		tex=mat.get_attribute(@@dict,'front_fresnel_scale_map',@@front_fresnel_scale)
		return IndigoTexture.new(tex)
	end

	def front_fresnel_scale_map=(tex)
		mat.set_attribute(@@dict,'front_fresnel_scale_map',tex.to_s)
	end

	def front_fresnel_scale_map?
		t=self.type
		case t
		when "Double-Sided Thin", "Simple Thin"
			return texture?(self.front_fresnel_scale_map)
		else
			return false
		end
	end

	def front_fresnel_scale_shader?

		t=self.type
		case t
		when "Double-Sided Thin", "Simple Thin"
			return self.front_fresnel_scale_map.procedural?
		else
			return false
		end
	end

	######
	def back_fresnel_scale_map
		tex=mat.get_attribute(@@dict,'back_fresnel_scale_map',@@back_fresnel_scale)
		return IndigoTexture.new(tex)
	end

	def back_fresnel_scale_map=(tex)
		mat.set_attribute(@@dict,'back_fresnel_scale_map',tex.to_s)
	end

	def back_fresnel_scale_map?
		t=self.type
		case t
		when "Double-Sided Thin", "Simple Thin"
			return texture?(self.back_fresnel_scale_map)
		else
			return false
		end
	end

	def back_fresnel_scale_shader?

		t=self.type
		case t
		when "Double-Sided Thin", "Simple Thin"
			return self.back_fresnel_scale_map.procedural?
		else
			return false
		end
	end

	######
	def r_f_map
		tex=mat.get_attribute(@@dict,'r_f_map',@@r_f)
		return IndigoTexture.new(tex)
	end

	def r_f_map=(tex)
		mat.set_attribute(@@dict,'r_f_map',tex.to_s)
	end

	def r_f_map?
		t=self.type
		case t
		when "Double-Sided Thin", "Simple Thin"
			return texture?(self.r_f_map)
		else
			return false
		end
	end

	def r_f_shader?

		t=self.type
		case t
		when "Double-Sided Thin", "Simple Thin"
			return self.r_f_map.procedural?
		else
			return false
		end
	end
	######
	def fresnel_map
		tex=mat.get_attribute(@@dict,'fresnel_map',@@fresnel_map)
		return IndigoTexture.new(tex)
	end

	def fresnel_map=(tex)
		mat.set_attribute(@@dict,'fresnel_map',tex.to_s)
	end

	def fresnel_map?
		t=self.type
		case t
		when "Coating", "Phong", "Fast SSS"
			return texture?(self.fresnel_map)
		else
			return false
		end
	end

	def fresnel_shader?

		t=self.type
		case t
		when "Coating", "Phong", "Fast SSS"
			return self.fresnel_map.procedural?
		else
			false
		end
	end
	######
	def thickness_map
		tex=mat.get_attribute(@@dict,'thickness_map',@@thickness_map)
		return IndigoTexture.new(tex)
	end

	def thickness_map=(tex)
		mat.set_attribute(@@dict,'thickness_map',tex.to_s)
	end

	def thickness_map?
		t=self.type
		case t
		when "Coating"
			return texture?(self.thickness_map)
		else
			return false
		end
	end

	def thickness_shader?

		t=self.type
		case t
		when "Coating"
			return self.thickness_map.procedural?
		else
			false
		end
	end
	def exponent_map
		tex=mat.get_attribute(@@dict,'exp_map',@@exp_map)
		return IndigoTexture.new(tex)
	end

	def exponent_map=(tex)
		mat.set_attribute(@@dict,'exp_map',tex.to_s)
	end

	def exponent_map?
		t=self.type
		case t
		when "Diffuse","Specular","Blend","Thin Glass","RGB Light","Blackbody Light","Exit Portal","Linked IGM"
			return false
		else
			return texture?(self.exponent_map)
		end
	end

	def exponent_shader?
		t=self.type
		case t
		when "Diffuse","Specular","Blend","Thin Glass","RGB Light","Blackbody Light","Exit Portal","Linked IGM"
			return false
		else
			return self.exponent_map.procedural?
		end

	end

	def clip_map
		tex=mat.get_attribute(@@dict,'clip_map',@@clip_map)
		return IndigoTexture.new(tex)
	end

	def clip_map=(tex)
		mat.set_attribute(@@dict,'clip_map',tex.to_s)
	end

	def clip_map?
		return false if !use_clip?
		t=self.type
		case t
		when "RGB Light","Blackbody Light","Exit Portal","Linked IGM", "Thin Glass"
			return false
		else
			return texture?(self.clip_map)
		end
	end

	def clip_shader?
		t=self.type
		case t
		when "RGB Light","Blackbody Light","Exit Portal","Linked IGM", "Thin Glass"
			return false
		else
			return self.clip_map.procedural?
		end
	end

	def use_clip?
		value=mat.get_attribute(@@dict,'use_clip',@@use_clip)
		value=false unless self.clip_map.valid?
		return value
	end

	def use_clip=(value)
		mat.set_attribute(@@dict,'use_clip',value)
	end

	def a_name
		mat.get_attribute(@@dict,'matA',@@matA)
	end

	def a_name=(value)
		mat.set_attribute(@@dict,'matA',value)
	end

	def b_name
		mat.get_attribute(@@dict,'matB',@@matB)
	end

	def b_name=(value)
		mat.set_attribute(@@dict,'matB',value)
	end

	def substrate
		mat.get_attribute(@@dict,'substrate',@@substrate)
	end

	def substrate=(value)
		mat.set_attribute(@@dict,'substrate',value)
	end

	def blend_factor
		mat.get_attribute(@@dict,'blendfactor',@@blendfactor)
	end

	def blend_factor=(value)
		mat.set_attribute(@@dict,'blendfactor',value)
	end

	def nk_data
		def_nkdata = File.join(File.dirname(SkIndigo.get_indigo_path), "nkdata", "Al.nk")
		path = mat.get_attribute(@@dict, 'nkdata', def_nkdata)
		return NKdata.new(path)
	end

	def nk_data=(value) #value is an NKdata ojbect
		mat.set_attribute(@@dict,'nkdata',value.path)  #stores the path to the nkdata
	end

	def alpha=(value)
		mat.alpha=(value)
	end

	def alpha
		mat.alpha
	end

	def display_name
		mat.display_name
	end

	def color=(value)
		mat.color=(value)
	end	

	#wdp is a string description of a wavelength dependent parameter (eg. albedo, emission)
	def color(wdp=nil)
		return mat.color if wdp==nil
		t=self.albedo_texture if wdp=="albedo"
		t=self.emission_map if wdp=="emission"
		t=self.specular_map if wdp=="specular"
		t=self.abs_layer_trans_map if wdp=="absorption_layer_transmittance"
		t=self.absorption_map if wdp=="absorption"
		t=self.sss_map if wdp=="sss"
		t=self.g_spectrum_map if wdp=="g_spectrum"
		t=self.base_emission if wdp=="base_emission"
		t=self.transmittance_map if wdp=="transmittance"
		case t.type
			when "SketchUp"
				return self.absorption_color if wdp=="absorption"
				return self.color.to_a
			when "Constant"
				return t.color
			when "Texture"
				return nil
			when "Shader"
				return nil
		end
	end
	
	
	#####subdivision settings
	def view_dependent_subdivision?
		mat.get_attribute(@@dict,'view_dep_sub',@@view_dependent)
	end

	def view_dependent_subdivision=(value)
		mat.set_attribute(@@dict,'view_dep_sub',value)
	end

	def max_subdiv
		mat.get_attribute(@@dict,'max_subdiv',@@max_subdiv)
	end

	def max_subdiv=(value)
		mat.set_attribute(@@dict,'max_subdiv',value)
	end

	def pixel_threshold
		mat.get_attribute(@@dict,'pixel_threshold',@@pixel_threshold)
	end

	def pixel_threshold=(value)
		mat.set_attribute(@@dict,'pixel_threshold',value)
	end

	def curvature_threshold
		#return -1  #this feature currently disabled
		mat.get_attribute(@@dict,'curvature_threshold',@@curvature_threshold)
	end

	def curvature_threshold=(value)
		mat.set_attribute(@@dict,'curvature_threshold',value)
	end

	def layer
		mat.get_attribute(@@dict,'layer',@@layer)
	end

	def layer=(value)
		mat.set_attribute(@@dict,'layer',value.to_i)
	end

	def valid?
		materials=Sketchup.active_model.materials
		begin
			if materials[mat.name]
				return true
			else
				return false
			end
		rescue
			return false
		end
	end

	def color_picker?

		if mat.display_name=="SkIndigo Color Picker"
			return true
		end

		return false
	end

	def uv_helper?

		if mat.display_name=="SkIndigo UV Helper"
			return true
		end

		return false
	end

	def texture?(tex)

		case tex.type
			when "SketchUp"
				return true if mat.texture!=nil
			when "External"
				return true #if tex.valid?
			else
				return false
		end
	end

	
	def background_material?

		name=@mat.name
		if name[0..2]=="BG_"
			return true
		else
			return false
		end
	end

	####
	def emission_scale
		mat.get_attribute(@@dict,'emission_scale',@@emission_scale)
	end

	def emission_scale=(value)
		mat.set_attribute(@@dict,'emission_scale',value)
	end

	def emission_measure
		mat.get_attribute(@@dict,'emission_measure',@@emission_measure)
	end

	def emission_measure=(value)
		mat.set_attribute(@@dict,'emission_measure',value)
	end
	
	def emission_sampling_factor
		mat.get_attribute(@@dict,'emission_sampling_factor',@@emission_sampling_factor)
	end

	def emission_sampling_factor=(value)
		mat.set_attribute(@@dict,'emission_sampling_factor',value)
	end

	def backface_emit?
		mat.get_attribute(@@dict,'backface_emit',@@backface_emit)
	end

	def backface_emit=(value)
		mat.set_attribute(@@dict,'backface_emit',value)
	end
	
	def mask_name
		mat.get_attribute(@@dict,'mask_name',@@mask_name)
	end

	def mask_name=(value)
		mat.set_attribute(@@dict,'mask_name',value)
	end
end #Indigo material class end
