#SkIndigo Ruby code is copyright Dale Martens 2011, all rights reserved, and
#Glare Technologies Limited, 2014 - 
#-----------------------------------------------------------------------------


###indigo mesh class for binary exporting - based on code that nick made opensource
class IndigoMesh
	
	attr_reader :uv_pairs, :uid, :vert_pos_data, :vert_normal_data, :uv_data, :triangle_data
	attr_writer :uv_pairs, :uid, :vert_pos_data, :vert_normal_data, :uv_data, :triangle_data
	attr_accessor :flattened

	MAGIC_NUMBER = 5456751
	FORMAT_VERSION = 1

	@@dict = "indigo07_materials"
	@@max_subdiv = 0
	@@view_dependent = false
	@@pixel_threshold = 4.0
	@@curvature_threshold = 0.1
	@@sub_smooth = false
	@@normal_smoothing = true
	@@external_mesh_path = ""
	@@use_external = false
	@@uv_set = 0
	@@mask_name = ""

	def IndigoMesh.external_skp?(defn)
		use_external = defn.get_attribute(@@dict, 'use_external', @@use_external)
		path = defn.get_attribute(@@dict, 'external_mesh_path', @@external_mesh_path)
		return use_external && FileTest.exist?(path) && (path[-4..-1].downcase == ".skp")
	end
	
	def IndigoMesh.external_obj?(defn)
		use_external = defn.get_attribute(@@dict, 'use_external', @@use_external)
		path = defn.get_attribute(@@dict, 'external_mesh_path', @@external_mesh_path)
		return use_external && FileTest.exist?(path) && (path[-4..-1].downcase == ".obj")
	end
	
	

	def initialize(defn=nil)

		@uid = -1
		
		@flattened = false


		if defn
			@defn = defn  #store the component definition object

			# Build the name for the mesh.
			if @defn.class == Sketchup::Group
				@name = "group " + defn.name
			else 
				@name = "component " + defn.name
			end
			
			if defn.name[0..3] == "EXT_" and FileTest.exist?(defn.description)
				self.external_mesh_path = defn.description
				self.use_external = true
			end
		else
			@name = "Top level mesh"
		end

		#self.add_uvset(0)
		if @defn
			dict=@defn.attribute_dictionary('uvsets')
			if dict
				num_uvsets=dict.length
				#p "uv sets #{num_uvsets}"
			else
				num_uvsets=1
			end
		else
			num_uvsets=1
		end
		@num_uv_mappings=num_uvsets  #there will always be at least one

		@vert_pos_data = []  # 3 floats per position vector
		@vert_normal_data = [] # 3 floats per normal vector
		@triangle_data = [] # (uint32 vertex indices[3], uint32 uv indices[3], uint32 material index)
		@uv_data = []
		@uv0 = []
		@uv1 = []
		@uv2 = []
		@uv3 = []
		@uv_pairs = []   #array of UVs (2 floats)
		@su_materials = [] # Array of used Sketchup::Material objects.  Elements may be nil in the case where the global default or component material (which the IndigoInstance object knows) is to be used.
	end
	



	def uv_texture?
		@defn.get_attribute(@@dict,'uv_texture',false)
	end

	def uv_texture=(value)
		@defn.set_attribute(@@dict,'uv_texture',value)
	end

	def add_uvset(index)

		index=index.to_s
		@defn.set_attribute('uvsets',index,true)
	end

	def has_uvset(index)

		return true if index==0
		
		return false if !@defn

		index = index.to_s
		@defn.get_attribute('uvsets', index, false)
	end

	def is_valid()
		begin
			view_dependent_subdivision?()
			return true
		rescue
			#puts "rescued exception in is_valid(): #{$!}"
			return false
		end
	end


	#####subdivision settings
	def view_dependent_subdivision?
		@defn ? @defn.get_attribute(@@dict,'view_dep_sub', @@view_dependent) : @@view_dependent
	end

	def view_dependent_subdivision=(value)
		@defn.set_attribute(@@dict,'view_dep_sub',value)
	end

	def max_subdiv
		@defn ? @defn.get_attribute(@@dict,'max_subdiv',@@max_subdiv).to_i : @@max_subdiv
	end

	def max_subdiv=(value)
		@defn.set_attribute(@@dict,'max_subdiv',value)
	end

	def pixel_threshold
		@defn ? @defn.get_attribute(@@dict,'pixel_threshold',@@pixel_threshold) : @@pixel_threshold
	end

	def pixel_threshold=(value)
		@defn.set_attribute(@@dict,'pixel_threshold',value)
	end

	def curvature_threshold
		@defn ? @defn.get_attribute(@@dict,'curvature_threshold', @@curvature_threshold) : @@curvature_threshold
	end

	def curvature_threshold=(value)
		@defn.set_attribute(@@dict,'curvature_threshold',value)
	end

	def subdivision_smoothing?  
		@defn ? @defn.get_attribute(@@dict,'sub_smooth',@@sub_smooth) : @@sub_smooth
	end

	def subdivision_smoothing=(value)
		@defn.set_attribute(@@dict,'sub_smooth',value) 
	end

	
	
	def mask_name
		@defn ? @defn.get_attribute(@@dict,'mask_name',@@mask_name) : @@mask_name
	end

	def mask_name=(value)
		@defn.set_attribute(@@dict,'mask_name',value) 
	end
	
	
	def external_mesh_path
		@defn ? @defn.get_attribute(@@dict,'external_mesh_path', @@external_mesh_path) : @@external_mesh_path
	end

	def external_mesh_path=(value)
		@defn.set_attribute(@@dict,'external_mesh_path',value)
	end

	def use_external_skp?
		use_external=self.use_external?
		if use_external
			if self.external_mesh_path[-4..1]==".skp"
				return true
			end
		end
		return false
	end

	def use_external_obj?
		use_external=self.use_external?
		if use_external
			#p self.external_mesh_path
			#p self.name
			if self.external_mesh_path[-4..-1].downcase==".obj"
				return true
			end
		end
		return false
	end

	def use_external? 
		if FileTest.exist?(self.external_mesh_path)
			@defn.get_attribute(@@dict, 'use_external', @@use_external)
		else
			return false
		end
	end

	def use_external=(value)
		@defn.set_attribute(@@dict, 'use_external', value) 
	end

	#######
	def su_materials
		return @su_materials
	end

	def guid
		return @defn ? @defn.guid : "top-level"
	end

	def name
		return @name.gsub(/[<>]/, '*')
	end
	
	def top_level_geom_mesh?
		return definition.nil?
	end

	def definition
		return @defn
	end

	def definition=(value)
		@defn=value
	end

	def uv_set
		@defn.nil? ? "default" : @defn.get_attribute(@@dict,'uv_set',@@uv_set)
	end

	def uv_set=(value)
		@defn.set_attribute(@@dict,'uv_set',value)
	end


	def add_uvs(uv0,uv1,uv2,uv3)

		#todo - add multiple uv sets
		#uv0=uv0.collect {|uv| uv.to_a}
		#p uv0
		#TODO check if << is faster than concatenation
		#p uv0,uv1,uv2,uv3
		#@uv0.concat(uv0)
		#uv1 ? @uv1.concat(uv1) : @uv1.concat(uv0)
		#uv2 ? @uv2.concat(uv2) : @uv2.concat(uv0)
		#uv3 ? @uv3.concat(uv3) : @uv3.concat(uv0)

		uv0.each {|uv| normalize_uvs(uv)}
		uv1.each {|uv| normalize_uvs(uv)}
		uv2.each {|uv| normalize_uvs(uv)}
		uv3.each {|uv| normalize_uvs(uv)}

		has_uv1=!uv1.empty?
		has_uv2=!uv2.empty?
		has_uv3=!uv3.empty?

		#now interleave the uvsets
		uv0.each_index {|i|
			@uv0.push(uv0[i])
			@uv0.push(uv1[i]) if has_uv1
			@uv0.push(uv2[i]) if has_uv2
			@uv0.push(uv3[i]) if has_uv3
		}

		#uv1 ? @uv0.concat(uv1) : @uv0.concat(uv0)
		#uv2 ? @uv0.concat(uv2) : @uv0.concat(uv0)
		#uv3 ? @uv0.concat(uv3) : @uv0.concat(uv0)
	end

	def normalize_uvs(uvs)

		uvz=uvs.z
		uvs.x=uvs.x/uvz
		uvs.y=uvs.y/uvz
	end
	
	def add_material(mat)
		mat_index = @su_materials.index(mat)
		
		if mat_index == nil
			mat_index = @su_materials.size
			@su_materials.push(mat)
		end
		
		return mat_index
	end
	

	####writing to file

	def writeBasic(out,object)   #object is a binary string
		out.write(object)
	end

	def write_string(out,string)
		# Indigo expects the size of the string, measured in bytes, to be written first.  Note that this is not the same as the number of characters, if there are unicode characters in the string.
		# Old versions of ruby (pre 1.8.7?) such as found in SketchUp 8 don't have bytesize.
		# Apparently in those versions length returned the bytesize anyway. (https://github.com/kjvarga/sitemap_generator/issues/7)
		len = HAVE_BYTESIZE ? string.bytesize : string.length
		len_bin = [len].pack("I") # Convert uint32
		writeBasic(out, len_bin) # Write the length of the string

		out.write(string)
	end

	def write_uint(out,uint)
		uint_bin = [uint].pack("I")  #convert to unsigned integer

		writeBasic(out,uint_bin)
	end

	def write_array(out, arr, type, vector_dim)
		len = arr.length    #get the number of items in the array
		num_vec_elems = len / vector_dim
		len_bin = [num_vec_elems].pack("I")  # Convert to uint32
		writeBasic(out, len_bin)  # Write the length of the array
		
		case type
		when :float
			data = arr.pack("f" + len.to_s)
			out.write(data)
		when :int
			data = arr.pack("l" + len.to_s)
			out.write(data)
		when :string
			arr.each {|mat| write_string(out,mat)}
		end
	end
		
	def write_to_file(path)
		out = File.new(path, "wb") # Create a new file in binary write mode
		
		begin
			write_uint(out, MAGIC_NUMBER)
			write_uint(out, FORMAT_VERSION)
			write_uint(out, 1) # TEMP: multiple UV sets support is disabled for now.    @num_uv_mappings)
			
			write_uint(out, 0) # Just write a zero-sized array for the used_materials, as are not used any more.
			write_uint(out, 0) # Just write a zero-sized array for the UV set expositions, as are not used any more.
			
			write_array(out, @vert_pos_data, :float, 3)
			#puts "wrote vert positions (#{@vert_pos_data.size} elems, #{@vert_pos_data.size/3} vertices)"
			write_array(out, @vert_normal_data, :float, 3)
			#puts "wrote vert normals (#{@vert_normal_data.size} elems, #{@vert_normal_data.size/3} normals)"
			
			write_array(out, @uv_data, :float, 2)
			#puts "wrote uvs (#{@uv_data.size} elems, #{@uv_data.size/2} uvs)"
			write_array(out, @triangle_data, :int, 7)
			#puts "wrote triangles (#{@triangle_data.size} elems, #{@triangle_data.size/7} triangles)"
		ensure
			out.close()
		end
	end

end #class indigomesh
