#SkIndigo Ruby code is copyright Glare Technologies Limited, 2014, all rights reserved
#-----------------------------------------------------------------------------
#

def reinterpret_as_utf8(str)
	if String.method_defined?(:encode)
		str = str.encode('UTF-16', 'UTF-8', :invalid => :replace, :replace => '') # Convert to UTF-16
		str = str.encode('UTF-8', 'UTF-16') # Convert back to UTF-8
	end
	return str
end

class IndigoScatterEditor

	def initialize(scatter_settings_, the_scatter, scatters_editor) 
	
		@scatter_settings = scatter_settings_
		@the_scatter = the_scatter
		@scatters_editor = scatters_editor
		@def_names = nil
		
		@dlg = UI::WebDialog.new("SkIndigo Scatter Editor", true, "SkIndigoScatterEditor", 630,650,419,243, true) #width,height,left,top
		@dlg.set_file(File.join(File.dirname(reinterpret_as_utf8(__FILE__)), "ScatterEditor.html"))
		color = @dlg.get_default_dialog_color
		@dlg.set_background_color(color)
		
		@dlg.add_action_callback("loaded") {|d,p|
			on_loaded
		}
		
		@dlg.add_action_callback("ValueChanged") {|d,p|
			#puts "ValueChanged()"
			
			pair = p.split("=")
			id = pair[0]
			value = pair[1]
		
			#puts "id: #{id}, value: #{value}"
			
			if id == "prototype-object"
				@the_scatter.prototype_def_name = @def_names[value.to_i]
				#puts "set prototype def to #{@the_scatter.prototype_def_name}"
			elsif id == "target-object"
				@the_scatter.target_def_name = @def_names[value.to_i]
				#puts "set target def to #{@the_scatter.target_def_name}"
			elsif id == "density"
				@the_scatter.density = value.to_f
			elsif id == "seed"
				@the_scatter.seed = value.to_i
			elsif id == "view-culling"
				@the_scatter.view_culling = value == "true"
			elsif id == "orient-normal"
				@the_scatter.orient_normal = value == "true"
			elsif id == "scale_std_dev"
				@the_scatter.scale_std_dev = value.to_f
			elsif id == "height_std_dev"
				@the_scatter.height_std_dev = value.to_f
			elsif id == "evenness"
				@the_scatter.evenness = value.to_f
			elsif SkIndigo.has_prefix?(id, "prototype-object-")
				group_index = SkIndigo.remove_prefix(id, "prototype-object-").to_i
				@the_scatter.groups[group_index].prototype_def_name = @def_names[value.to_i]
			elsif SkIndigo.has_prefix?(id, "weight-")
				group_index = SkIndigo.remove_prefix(id, "weight-").to_i
				@the_scatter.groups[group_index].weight = value.to_f
			end
			
			@scatter_settings.valuesChanged()
			@scatters_editor.scatterChanged()
		}
		
		@dlg.add_action_callback("deleteGroup") {|d,p|
			pair = p.split("=")
			id = pair[0]
			value = pair[1]
			
			on_delete_group(value.to_i)
		}
		
		@dlg.add_action_callback("addNewGroup") {|d,p|
			on_add_new_group
		}
	end

	def show
		@dlg.show
	end

	def on_loaded()

		# populate drop-down boxes
		@def_names = Sketchup.active_model.definitions.collect { |d| d.name }.to_a
		
		#puts "def names: #{def_names.inspect}"
		
		# Definition names can have all sorts of characters in them (single quotes etc..) that can cause problems.
		# So just set the values to the be the indices of the names.
		index = 0
		name_values_array = @def_names.map do |n, i|
			pair = [n, index.to_s]
			index += 1
			pair
		end
		#puts "name_values_array: #{name_values_array.inspect}"
		
		
		# Add object groups
		
		# Add an object group if there are none existing.
		if @the_scatter.groups.empty?
			@the_scatter.groups << ProtoObjectGroup.new
		end
		
		@dlg.execute_script("clearGroups()")
		index = 0
		@the_scatter.groups.each do |group|
			@dlg.execute_script("addGroup(" + index.to_s + ")")		
			
			SkIndigo.set_dropdown_options(@dlg, "prototype-object-#{index}", name_values_array)
			
			
			setValue("prototype-object-#{index}", @def_names.index(group.prototype_def_name))
			if group.prototype_def_name && !group.prototype_def_name.empty?
				cmd = "$(\"#prototype-object-#{index}\").removeClass(\"invalid\")"
				@dlg.execute_script(cmd)
			end
			
			setValue("weight-#{index}", group.weight)
			
			index += 1
		end
			
		
		
		#SkIndigo.set_dropdown_options(@dlg, "prototype-object", name_values_array)
		SkIndigo.set_dropdown_options(@dlg, "target-object",    name_values_array)
		
		
		#setValue("prototype-object", @def_names.index(@the_scatter.prototype_def_name))
		setValue("target-object", @def_names.index(@the_scatter.target_def_name))
		if @the_scatter.target_def_name && !@the_scatter.target_def_name.empty?
			@dlg.execute_script("$(\"#target-object\").removeClass(\"invalid\")")
		end
		setValue("density", @the_scatter.density)
		setValue("seed", @the_scatter.seed)
		setCheckbox("view-culling", @the_scatter.view_culling)
		setCheckbox("orient-normal", @the_scatter.orient_normal)
		setValue("scale_std_dev", @the_scatter.scale_std_dev)
		setValue("height_std_dev", @the_scatter.height_std_dev)
		setValue("evenness", @the_scatter.evenness)
	end
	
	def setCheckbox(id, checked)
		SkIndigo.set_checkbox(@dlg, id, checked)
	end

	def setValue(id, value)
		SkIndigo.set_value(@dlg, id, value)
	end
	
	def on_delete_group(index)
		#puts "on_delete_group(), index=#{index}"
	
		@the_scatter.groups.delete_at(index)
	
		@scatter_settings.save_scatter_settings
		
		on_loaded() # Re-layout widget
	end
	
	def on_add_new_group()
		#puts  "on_add_new_group()"
	
		group = ProtoObjectGroup.new
		@the_scatter.groups << group
		
		#puts "@the_scatter: #{@the_scatter.inspect}"
		
		@scatter_settings.save_scatter_settings
		
		on_loaded() # Re-layout widget
	end


end

