#SkIndigo Ruby code is copyright Dale Martens 2011, all rights reserved, and
#Glare Technologies Limited, 2014 - 
#-----------------------------------------------------------------------------

##########################################Render Settings Editor Class

def reinterpret_as_utf8(str)
	if String.method_defined?(:encode)
		str = str.encode('UTF-16', 'UTF-8', :invalid => :replace, :replace => '') # Convert to UTF-16
		str = str.encode('UTF-8', 'UTF-16') # Convert back to UTF-8
	end
	return str
end

class IndigoSettingsEditor

	def initialize
		@dlg=UI::WebDialog.new("SkIndigo Render Settings", false, # scrollable
			"SkIndigoRenderSettings", # pref_key 
			700, # width
			600, # height
			370, # left
			180, # top
			true
		)
		@dlg.set_file(File.join(File.dirname(reinterpret_as_utf8(__FILE__)),"Settings.html"))
		color = @dlg.get_default_dialog_color 
		@dlg.set_background_color(color)
		@irs=IndigoRenderSettings.new
		Sketchup::Pages.add_frame_change_observer(SkIndigoFrameChangeObserver.new)
		
		@on_mac=SkIndigo.on_mac?
		@dlg.add_action_callback("apply_settings") {|d,p|
			@irs.save_to_model()
			apply_settings(d,p)
			
		} #end apply settings callback
		@dlg.add_action_callback("apply_to_scene") {|d,p|
			
			pages=Sketchup.active_model.pages
			if pages.count>0
				page=pages.selected_page
				if page==nil
					page=pages[0]
				end
				@irs.save_to_scene(page)
				apply_settings(d,p)
			else
				UI.messagebox("You must first select a SketchUp scene.")
			end
		}
		@dlg.add_action_callback("getViewSize") {|d,p|
			width=(Sketchup.active_model.active_view.vpwidth).to_s
			height=(Sketchup.active_model.active_view.vpheight).to_s
			cmd="document.getElementById('width').value=\"#{width}\""
			d.execute_script(cmd)
			cmd="document.getElementById('height').value=\"#{height}\""
			d.execute_script(cmd)
		} #end get View Size callback
		@dlg.add_action_callback("ValueChanged") {|d,p|
			pair=p.split("=")
			id=pair[0]
			value=pair[1]
			case id
				when "inifile"
					Indigo.adv   #opens inifile.xml for editing advanced indigo settings
				when "pickFocalPoint"
					Sketchup.active_model.select_tool IndigoFocusTool.new     #launches to focus tool
					setCheckbox("autofocus","false")
				when "envMapBrowser"
					if SkIndigo.sketchup_is_2014_or_later # Multiple file type wildcards were broken before SketchUp 2014: http://www.sketchup.com/intl/en/developer/docs/ourdoc/ui?csw=1#openpanel
						exr = UI.openpanel("Select Environment Map", "", "Image file (*.exr;*.hdr;*.jpg;*.jpeg;*.png;*.tif;*.tiff;*.tga;*.gif;*.bmp)|*.exr;*.hdr;*.jpg;*.jpeg;*.png;*.tif;*.tiff;*.tga;*.gif;*.bmp||")
					else
						exr = UI.openpanel("Select Environment Map", "", "Image file (.exr;.hdr;.jpg;.jpeg;.png;.tif;.tiff;.tga;.gif;.bmp)")
					end
					if exr
						@irs.store_texture(exr)
						mats=Sketchup.active_model.materials
						mat=mats.add("EnvironmentMap")
						mat=IndigoMaterial.new(mat)
						mat.layer=0
						bm=mat.base_emission
						bm.uni_value=1.0
						bm.spectrum_type="Uniform"
						mat.base_emission=bm
						mat.emission_scale=75000
						mat.emission_measure="luminous_emittance"
						em=mat.emission_map
						em.a=0.0
						em.b=1.0
						em.c=0.0
						em.exponent=1.0
						em.type="External"
						em.path=exr
						#em.path=exr.gsub!("\\","\/")
						mat.emission_map=em
						mat.use_emission=true
						refreshMaterialsList()
						setValue("MaterialName",mat.name)
						setRadioButton("EnvironmentMap","true")
						
					end
				when "apertureImageBrowser"
					path=UI.openpanel "Select Aperture Map", "", "*.png"
					if path
						path.gsub!("\\","\/")
						setValue("apertureImage",path)
					end
				when "obstacleMapBrowser"
					path=UI.openpanel "Select Obstacle Map", "", "*.png"
					if path
						path.gsub!("\\","\/")
						setValue("obstacleMap",path)
					end
			end
		} # end Value Changed callback
		@dlg.add_action_callback("cancel_changes") {|d,p|
			loadValuesFromSU()
		} 
		@dlg.add_action_callback("reset_camera") {|d,p|
			current_camera=Sketchup.active_model.active_view.camera
			current_aspect=current_camera.aspect_ratio
			if current_aspect!=0.0
				current_camera.aspect_ratio=0.0
				current_camera.image_width=0.0
				current_camera.focal_length=current_camera.focal_length*(current_aspect)
			end
		}   
		@dlg.add_action_callback("undo_changes") {|d,p|
			Sketchup.undo
			loadValuesFromSU()
			model=Sketchup.active_model
			view=model.active_view
			width=@irs.width  
			height=@irs.height
			current_camera=model.active_view.camera
			old_aspect=current_camera.aspect_ratio
			old_aspect=1.0 if old_aspect==0.0
			f=current_camera.focal_length
			current_camera.aspect_ratio=(width.to_f)/(height.to_f)
			current_camera.focal_length=f/(current_camera.aspect_ratio/old_aspect)
		}
		@dlg.add_action_callback("select_region") {|d,p|
			validate_render_region()
			#@irs.render_region=true
			Sketchup.active_model.select_tool SelectRegionTool.new
			setCheckbox("regionRenderCheckbox","true")
		}
		@dlg.add_action_callback("set_sun") {|d,p|
			#@irs.sun_pos_override=true
			Sketchup.active_model.select_tool SkIndigoSunTool.new
			setCheckbox("overrideSunCheckbox","true")
		}

		@dlg.add_action_callback("happy_easter") {|d,p|
			SkIndigo.export_animation(false)
			#SkIndigo.export_animation(true)   #sketchy physics testing
		} 
		@dlg.add_action_callback("refresh") {|d,p|
			refresh_dialog()
		}
		@dlg.add_action_callback("save_defaults") {|d,p|
			begin
				status=@irs.store_settings()
				if status
					UI.messagebox("Current render settings successfully saved as defaults.")
				else
					UI.messagebox("You must first apply your current settings.")
				end
			rescue
				UI.messagebox("Unable to save settings.")
				raise
			end
		}
		
	end #initialize

	
	def read_setting(param_map, setting_name)
		val = param_map[setting_name]
		if(val.nil?)
			puts "WARNING: Failed to find setting #{setting_name} in param map"
		end
		return val
	end
		
	def setting_to_bool(setting_string_val)
		if(setting_string_val == "true")
			return true
		elsif(setting_string_val == "false")
			return false
		else
			puts "Warning: invalid setting_string_val: #{setting_string_val.inspect}"
		end
	end
	
	
	def apply_settings(d,p)  #d is the dialog, p is string of parameters
			
			#extract checkbox and radio values
			# NOTE: We have to pass checkbox (and radio?) values this way because d.get_element_value() is buggy - it always returns "on" even if a checkbox is unchecked.
			
			# Split params string and store keys and values in a map.
			params = p.split("|")
			param_map = {}
			params.each do |s|
				key   = s.split("=")[0]
				value = s.split("=")[1]
				param_map[key] = value
			end
						
			save_igi=params[0].split("=")[1] 
			custom_name=params[1].split("=")[1]
			tm=params[2].split("=")[1]
			af=params[3].split("=")[1]
			env=params[4].split("=")[1]
			auto_threads=params[5].split("=")[1]
			low_priority=params[6].split("=")[1]
			watermark=params[7].split("=")[1]
			ad=params[8].split("=")[1]
			om=params[9].split("=")[1]
			rr=params[10].split("=")[1]
			io=params[11].split("=")[1]
			so=params[12].split("=")[1]
			ga=params[13].split("=")[1]
			vi=params[14].split("=")[1]
			bgs=params[15].split("=")[1]
			la=params[16].split("=")[1]
			gpu=params[17].split("=")[1]
			sky_model=params[18].split("=")[1]
			cull=params[19].split("=")[1]
			alpha=params[20].split("=")[1]
			
			model=Sketchup.active_model
			model.start_operation "Render Settings"

			@irs.watermark=true if (watermark=="true")
			@irs.watermark=false if (watermark=="false")
			@irs.save_igi=true if (save_igi=="true")
			@irs.save_igi=false if (save_igi=="false")
			@irs.custom_image_name=true if (custom_name=="true")
			@irs.custom_image_name=false if (custom_name=="false")
			@irs.tm=tm
			@irs.focal_point="auto" if (af=="true")
			@irs.environment=env
			@irs.auto_choose_threads=true if (auto_threads=="true")
			@irs.auto_choose_threads=false if (auto_threads=="false")
			@irs.low_priority=true if (low_priority=="true")
			@irs.low_priority=false if (low_priority=="false")
			@irs.aperture_diffraction=true if (ad=="true")
			@irs.aperture_diffraction=false if (ad=="false")
			@irs.use_obstacle_map=true if (om=="true")
			@irs.use_obstacle_map=false if (om=="false")
			@irs.render_region=true if (rr=="true") and (validate_render_region())
			@irs.render_region=false if (rr=="false")
			@irs.info_overlay=true if (io=="true")
			@irs.info_overlay=false if (io=="false")
			@irs.sun_pos_override=true if (so=="true")
			@irs.sun_pos_override=false if (so=="false")
			#@irs.glass_acceleration=true if (ga=="true")
			#@irs.glass_acceleration=false if (ga=="false")
			@irs.vignetting=true if (vi=="true")
			@irs.vignetting=false if (vi=="false")
			@irs.bg_sun=true if (bgs=="true")
			@irs.bg_sun=false if (bgs=="false")
			@irs.lock_aspect=true if (la=="true")
			@irs.lock_aspect=false if (la=="false")
			@irs.use_gpu=(gpu=="true") ? true : false
			@irs.sky_model=(sky_model=="true") ? "original" : "captured-simulation"
			@irs.cull_geometry=(cull=="true") ? true : false
			@irs.foreground_alpha=(alpha=="true") ? true : false
			
			@irs.width=d.get_element_value("width")
			@irs.height=d.get_element_value("height")
			@irs.image_name=d.get_element_value("image_name")
			@irs.aperture=d.get_element_value("aperture")
			@irs.white_balance=d.get_element_value("wb")
			@irs.turbidity=d.get_element_value("turbidity")
			@irs.method=d.get_element_value("method")
			#@irs.mncr=d.get_element_value("MNCR")
			@irs.world_scale=d.get_element_value("worldScale")
			@irs.image_save_period=d.get_element_value("savePeriod")
			@irs.halt=d.get_element_value("halt")
			@irs.halt_unit=d.get_element_value("haltUnit")
			@irs.num_threads=d.get_element_value("threads")
			#@irs.thumbnail_path=d.get_element_value("thumbPath")
			@irs.pre_scale=d.get_element_value("preScale")
			@irs.post_scale=d.get_element_value("postScale")
			@irs.burn=d.get_element_value("burn")
			@irs.iso=d.get_element_value("iso")
			@irs.shutter_speed=d.get_element_value("shutterSpeed")
			@irs.ev_adjust=d.get_element_value("exposure")
			@irs.linear_scale = d.get_element_value("linearScale")
			@irs.filmic_scale = d.get_element_value("filmicScale")
			#@irs.exr=d.get_element_value("exr")
			#@irs.exr_type=d.get_element_value("envMapType")
			@irs.exr_shift=d.get_element_value("envMapRotation")
			#@irs.exr_width=d.get_element_value("envMapWidth")
			#@irs.exr_gain=d.get_element_value("envMapGain")
			@irs.aperture_shape=d.get_element_value("apertureShape")
			@irs.blade_offset=d.get_element_value("bladeOffset")
			@irs.blade_curvature_radius=d.get_element_value("curvatureRadius")
			@irs.start_angle=d.get_element_value("startAngle")
			@irs.aperture_image=d.get_element_value("apertureImage")
			@irs.obstacle_map=d.get_element_value("obstacleMap")
			@irs.shift_up=d.get_element_value("shiftUp")
			@irs.shift_right=d.get_element_value("shiftRight")
			@irs.supersample_factor=d.get_element_value("supersampleFactor")
			@irs.sun_layer=d.get_element_value("sunLayer")
			@irs.sky_layer=d.get_element_value("skyLayer")
			@irs.camera_type=d.get_element_value("cameraType")
			#@irs.env_map_layer=d.get_element_value("envMapLayer")
			#@irs.frame_upload_period=d.get_element_value("frameUploadPeriod")
			@irs.network_mode=d.get_element_value("networkMode")
			@irs.cam_resp_func=d.get_element_value("responseFunction")
			#@irs.bg_luminance=d.get_element_value("envMapGain")
			@irs.bg_weight=d.get_element_value("bgWeight")
			@irs.sunsky_weight=d.get_element_value("sunskyWeight")
			@irs.bg_material=d.get_element_value("MaterialName")
			
			@irs.clamp_contributions = setting_to_bool(read_setting(param_map, "clamp_contributions"))
			@irs.max_contribution = d.get_element_value("maxContribution")
			@irs.gpu_max_depth = d.get_element_value("gpuMaxDepth")
			
			@irs.save_channels	 						= setting_to_bool(read_setting(param_map, "saveChannels"))
			
			@irs.normals_channel 						= setting_to_bool(read_setting(param_map, "normals_channel"))
			@irs.normals_pre_bump_channel 				= setting_to_bool(read_setting(param_map, "normals_pre_bump_channel"))
			@irs.position_channel 						= setting_to_bool(read_setting(param_map, "position_channel"))
			@irs.depth_channel 							= setting_to_bool(read_setting(param_map, "depth_channel"))
			@irs.material_id_channel 					= setting_to_bool(read_setting(param_map, "material_id_channel"))
			@irs.object_id_channel						= setting_to_bool(read_setting(param_map, "object_id_channel"))
			@irs.foreground_channel 					= setting_to_bool(read_setting(param_map, "foreground_channel"))
			@irs.material_masks 						= setting_to_bool(read_setting(param_map, "material_masks"))
			@irs.object_masks 							= setting_to_bool(read_setting(param_map, "object_masks"))
			
			@irs.direct_lighting_channel 				= setting_to_bool(read_setting(param_map, "direct_lighting_channel"))
			@irs.indirect_lighting_channel 				= setting_to_bool(read_setting(param_map, "indirect_lighting_channel"))
			@irs.specular_reflection_lighting_channel 	= setting_to_bool(read_setting(param_map, "specular_reflection_lighting_channel"))
			@irs.refraction_lighting_channel 			= setting_to_bool(read_setting(param_map, "refraction_lighting_channel"))
			@irs.transmission_lighting_channel 			= setting_to_bool(read_setting(param_map, "transmission_lighting_channel"))
			@irs.emission_lighting_channel 				= setting_to_bool(read_setting(param_map, "emission_lighting_channel"))
			@irs.participating_media_lighting_channel 	= setting_to_bool(read_setting(param_map, "participating_media_lighting_channel"))
			@irs.sss_lighting_channel 					= setting_to_bool(read_setting(param_map, "sss_lighting_channel"))
		
			#p @irs.bg_material
							
			current_camera=model.active_view.camera
			old_aspect=current_camera.aspect_ratio
			old_aspect=1.0 if old_aspect==0.0
			f=current_camera.focal_length
			current_camera.aspect_ratio=(@irs.width.to_f)/(@irs.height.to_f)
			current_camera.focal_length=f/(current_camera.aspect_ratio/old_aspect)
			
			model.commit_operation
	end

	def refresh_dialog(page=nil)
		refreshCamResponseList()
		refreshLayerList()
		refreshMaterialsList()
		loadValuesFromSU(page)
		setApertureVisible()
		#setExrWidth()
	end

	def show
		if @dlg.visible?
			self.refresh_dialog()
		else
			if @on_mac
				self.refresh_dialog()
				@dlg.show_modal{}
			else
				@dlg.show{}
			end
		end
	end

	def refreshMaterialsList()

		mat_list=Sketchup.active_model.materials.collect {|m| m.name}
		mat_list.sort!

		options = mat_list.map { |mat_name| [mat_name, mat_name] }
		SkIndigo.set_dropdown_options(@dlg, "MaterialName", options)
	end

	def validate_render_region()
		if @irs.tm=="Reinhard"   #region rendering only works with camera or linear tonemapping
			UI.messagebox "Tonemapping has been set to 'Camera' for use with region rendering."
			@irs.tm="Camera"
			setRadioButton("cameraTonemapping","true")
		end
		return true
	end

	def setExrWidth()
		#@dlg.execute_script("setExrWidthVisible()")
	end

	def select_region_tool()
		#Sketchup.active_model.select_tool(SelectRegionTool.new)
	end

	def setApertureVisible()
		@dlg.execute_script("setApertureAttributesVisible()")
	end

	def loadValuesFromSU(page=nil)
		#check if current scene contains indigo settings

		#disabling loading from scene due to bugs
		#if page!=nil
		#	@irs.save_to_scene(page)   #this will cause the settings stored to the current scene to be loaded
			#p "load froms scene"
		#else
		#	@irs.save_to_model()
			#p "load from model"
		#end
		#set checkboxes and radio buttons
		env=@irs.environment
		tm=@irs.tm
		
		if @irs.bg_sun? then setCheckbox("bgSun","true")
		else setCheckbox("bgSun","false")
		end
		if @irs.vignetting? then setCheckbox("vignetting","true")
		else setCheckbox("vignetting","false")
		end
		#if @irs.glass_acceleration? then setCheckbox("glassAccel","true")
		#else setCheckbox("glassAccel","false")
		#end
		if @irs.watermark? then setCheckbox("watermark","true")
		else setCheckbox("watermark","false")
		end
		if @irs.sun_pos_override? then setCheckbox("overrideSunCheckbox","true")
		else setCheckbox("overrideSunCheckbox","false")
		end
		if @irs.render_region? then setCheckbox("regionRenderCheckbox","true")
		else setCheckbox("regionRenderCheckbox","false")
		end
		if @irs.info_overlay? then setCheckbox("infoOverlay","true")
		else setCheckbox("infoOverlay","false")
		end
		if @irs.save_igi? then setCheckbox("saveigi","true")
		else setCheckbox("saveigi","false")
		end
		if @irs.save_channels? then setCheckbox("saveChannels","true")
		else setCheckbox("saveChannels","false")
		end
		if @irs.custom_image_name? then setCheckbox("customName","true")
		else setCheckbox("customName","false")
		end
		if @irs.focal_point=="auto" then setCheckbox("autofocus","true")
		else setCheckbox("autofocus","false")
		end
		if env=="Sun and Sky" then setRadioButton("SunSky","true")
		elsif env=="Background" then setRadioButton("Background","true")
		elsif env=="Black" 
			setRadioButton("Background","true")
			setCheckbox("black","true")
		elsif env=="Environment Map" then setRadioButton("EnvironmentMap","true")
		end
		if tm=="Reinhard" then setRadioButton("Reinhard","true")
		elsif tm=="Camera" then setRadioButton("cameraTonemapping","true")
		elsif tm=="Linear" then setRadioButton("Linear","true")
		elsif tm=="Filmic" then setRadioButton("Filmic","true")
		end
		if @irs.auto_choose_threads? then setCheckbox("autoThreads","true")
		else setCheckbox("autoThreads","false")
		end
		if @irs.low_priority? then setCheckbox("lowPriority","true")
		else setCheckbox("lowPriority","false")
		end
		if @irs.aperture_diffraction? then setCheckbox("apertureDiffractionCheckbox","true")
		else setCheckbox("apertureDiffractionCheckbox","false")
		end
		if @irs.use_obstacle_map? then setCheckbox("obstacleCheckbox","true")
		else setCheckbox("obstacleCheckbox","false")
		end
		if @irs.lock_aspect? then setCheckbox("lockAspect","true")
		else setCheckbox("lockAspect","false")
		end
		if @irs.use_gpu? then setCheckbox("useGPU","true")
		else setCheckbox("useGPU","false")
		end
		setCheckbox("cullGeometry",@irs.cull_geometry?)
		if @irs.method=="Alpha"
			@irs.foreground_alpha=true
			@irs.method="Bidir PT"
		end
		setCheckbox("foregroundAlpha",@irs.foreground_alpha?)
		setCheckbox("clampContributions",@irs.clamp_contributions?)
		
		if @irs.sky_model=="original" then setCheckbox("skyModel","true")
		else setCheckbox("skyModel","false")
		end
		
		
		setCheckbox("normals_channel", @irs.normals_channel?)
		setCheckbox("normals_pre_bump_channel", @irs.normals_pre_bump_channel?)
		setCheckbox("position_channel", @irs.position_channel?)
		setCheckbox("depth_channel", @irs.depth_channel?)
		setCheckbox("material_id_channel", @irs.material_id_channel?)
		setCheckbox("object_id_channel", @irs.object_id_channel?)
		setCheckbox("foreground_channel", @irs.foreground_channel?)
		setCheckbox("material_masks", @irs.material_masks?)
		setCheckbox("object_masks", @irs.object_masks?)
		setCheckbox("direct_lighting_channel", @irs.direct_lighting_channel?)
		setCheckbox("indirect_lighting_channel", @irs.indirect_lighting_channel?)
		setCheckbox("specular_reflection_lighting_channel", @irs.specular_reflection_lighting_channel?)
		setCheckbox("refraction_lighting_channel", @irs.refraction_lighting_channel?)
		setCheckbox("transmission_lighting_channel", @irs.transmission_lighting_channel?)
		setCheckbox("emission_lighting_channel", @irs.emission_lighting_channel?)
		setCheckbox("participating_media_lighting_channel", @irs.participating_media_lighting_channel?)
		setCheckbox("sss_lighting_channel", @irs.sss_lighting_channel?)
		
		#Set other dialog input values
		setValue("width",(@irs.width))  
		setValue("height",(@irs.height)) 
		setValue("image_name",(@irs.image_name)) 
		setValue("aperture",(@irs.aperture))
		setValue("wb",(@irs.white_balance))
		setValue("method",(@irs.method))
		setValue("worldScale",(@irs.world_scale))
		setValue("savePeriod",(@irs.image_save_period))
		setValue("halt",(@irs.halt))
		setValue("haltUnit",(@irs.halt_unit))
		#setValue("exr",(@irs.exr))
		#setValue("envMapType",(@irs.exr_type))
		#setValue("envMapWidth",(@irs.exr_width))
		setValue("imagePath",(@irs.image_path))
		setValue("thumbPath",@irs.thumbnail_path)
		setValue("apertureShape",@irs.aperture_shape)
		setValue("bladeOffset",@irs.blade_offset)
		setValue("curvatureRadius",@irs.blade_curvature_radius)
		setValue("startAngle",@irs.start_angle)
		setValue("apertureImage", @irs.aperture_image)
		setValue("obstacleMap", @irs.obstacle_map)
		setValue("shiftUp",@irs.shift_up)
		setValue("shiftRight",@irs.shift_right)
		setValue("supersampleFactor",@irs.supersample_factor)
		setValue("sunLayer",@irs.sun_layer)
		setValue("skyLayer",@irs.sky_layer)
		setValue("cameraType",@irs.camera_type)
		#setValue("envMapLayer",@irs.env_map_layer)
		setValue("networkMode",@irs.network_mode)
		#setValue("frameUploadPeriod",@irs.frame_upload_period)
		setValue("responseFunction",@irs.cam_resp_func)
		setValue("bgWeight",@irs.bg_weight)
		setValue("sunskyWeight",@irs.sunsky_weight)
		#p @irs.bg_material
		setValue("MaterialName",@irs.bg_material)
		setValue("gpuMaxDepth",@irs.gpu_max_depth)
		
		
		#Set Slider Values
		pre_scale=(@irs.pre_scale).to_f*10
		@dlg.execute_script("s.setValue(#{pre_scale.to_s})")
		post_scale=(@irs.post_scale).to_f*10
		@dlg.execute_script("s2.setValue(#{post_scale.to_s})")
		burn=(@irs.burn).to_f*10
		@dlg.execute_script("s3.setValue(#{burn.to_s})")
		@dlg.execute_script("s4.setValue(#{@irs.linear_scale.to_s})")
		iso=(@irs.iso)
		@dlg.execute_script("s5.setValue(#{iso})")
		shutterSpeed=(@irs.shutter_speed).to_f*10
		@dlg.execute_script("s6.setValue(#{shutterSpeed})")
		exposure=(@irs.ev_adjust).to_f*10
		@dlg.execute_script("s7.setValue(#{exposure})")
		turbidity=(@irs.turbidity).to_f*10
		@dlg.execute_script("s8.setValue(#{turbidity.to_s})")
		#envMapGain=(@irs.bg_luminance)
		#@dlg.execute_script("s9.setValue(#{envMapGain})")
		envMapRotation=(@irs.exr_shift)
		@dlg.execute_script("s10.setValue(#{envMapRotation})")
		threads=(@irs.num_threads)
		@dlg.execute_script("s11.setValue(#{threads})")
		max_contribution=(@irs.max_contribution)
		@dlg.execute_script("s12.setValue(#{max_contribution})")
		@dlg.execute_script("s13.setValue(#{@irs.filmic_scale.to_s})")
			
	end #loadValuesFromSU

	def setValue(id,value)
		new_value=value.to_s
		cmd="document.getElementById('#{id}').value=\"#{new_value}\""
		@dlg.execute_script(cmd)
	end

	def setCheckbox(id,checked)
		checked=checked.to_s  #can take either a true or false parameter as a string or TrueClass/FalseClass
		cmd="document.getElementById('#{id}').checked=#{checked}"
		@dlg.execute_script(cmd)
	end

	def setRadioButton(id,checked)
		cmd="document.getElementById('#{id}').checked=\"#{checked}\""
		@dlg.execute_script(cmd)
	end

	def dialog   
		return @dlg
	end

	def refreshCamResponseList()

		cmd="$(\"#responseFunction\").html(\"\")"
		@dlg.execute_script(cmd)
		func_list=@irs.get_camera_response_functions
		func_list=func_list.collect {|item| File.basename(item,".*")}
		#p func_list

		func_list.each {|func|
			cmd="$(\"<option>\").attr(\"value\",\"#{func}\").html(\"#{func}\").appendTo(\"#responseFunction\");"
			#p cmd
			@dlg.execute_script(cmd)
		}
	end

	def refreshLayerList()
		layer_ids=["sunLayer","skyLayer","envMapLayer"]
		layer_ids.each {|id|
			cmd="$(\"##{id}\").html(\"\")"
			@dlg.execute_script(cmd)
			layer_names=IndigoRenderSettings.new.light_layer_names
			layer_names.each_index {|i|
				cmd="$(\"<option>\").attr(\"value\",\"#{i}\").html(\"#{layer_names[i]}\").appendTo(\"##{id}\");"
				#p cmd
				@dlg.execute_script(cmd)
			}
		}
	end

end #class
