#SkIndigo Ruby code is copyright Dale Martens 2011, all rights reserved, and
#Glare Technologies Limited, 2014 - 
#-----------------------------------------------------------------------------

###################Indigo Texture Class################
##this class used to only represent textures, now it has been expanded to support more parameters including other spectrum types
class IndigoTexture

	include DMcolormath

	@@preview_uvset=0  #this is the uvset index that will be exported for material previews

	attr_reader :uv_set, :a, :b, :c, :exponent, :u_scale, :v_scale, :shader_code, :type, :path, :spectrum_type, :bb_temperature, :bb_gain, :rgb_r, :rgb_g, :rgb_b, :rgb_gain, :rgb_gamma, :reg_start, :reg_end, :reg_num_values, :reg_values, :uni_value, :smooth
	attr_writer :uv_set, :a, :b, :c, :exponent, :u_scale, :v_scale, :shader_code, :type, :path, :spectrum_type, :bb_temperature, :bb_gain, :rgb_r, :rgb_g, :rgb_b, :rgb_gain, :rgb_gamma, :reg_start, :reg_end, :reg_num_values, :reg_values, :uni_value, :smooth

	def initialize(attribute_string="Constant")    #The format of the attribute string must be "type|uv_set|a|b|c|exponent|shader_code|texture_path|spectrum_type|bb_temperature|bb_gain|rgb_r|rgb_g|rgb_b|rgb_multi|rgb_gamma|reg_start|reg_end|reg_num_values|reg_values|uniform_value"
		attributes=attribute_string.split("|")
		@type=attributes[0]
		case @type
		when "SketchUp","External","Shader"
			#do nothing
		when "None"
			@type="Constant"
		when "SU"
			@type="SketchUp"  #backward compat.
		else
			@type="Constant"  #same as constant
		end
		@uv_set=attributes[1] ? attributes[1]:"Default"
		@uv_set="Default" if @uv_set=="0"  #for backwards compatibility
		@a=attributes[2] ? attributes[2].to_f : 0.0
		@b=attributes[3] ? attributes[3].to_f : 1.0
		@c=attributes[4] ? attributes[4].to_f : 0.0
		@exponent=attributes[5] ? attributes[5]:1.0
		@shader_code=attributes[6] ? attributes[6]:"Enter Shader Code"
		@path=attributes[7] ? attributes[7]:""
		@spectrum_type=attributes[8] ? attributes[8]:"RGB"
		@bb_temperature=attributes[9] ? attributes[9]:5000
		@bb_gain=attributes[10] ? attributes[10]:1.0
		@rgb_r=attributes[11] ? attributes[11]:0.0
		@rgb_g=attributes[12] ? attributes[12]:0.0
		@rgb_b=attributes[13] ? attributes[13]:0.0
		@rgb_gain=attributes[14] ? attributes[14]:1.0
		@rgb_gamma=attributes[15] ? attributes[15]:2.2
		@reg_start=attributes[16] ? attributes[16]:385  #in nm
		@reg_end=attributes[17] ? attributes[17]:775  #in nm
		@reg_num_values=attributes[18] ? attributes[18]:10
		@reg_values=attributes[19] ? attributes[19]:"0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0"
		@uni_value=attributes[20] ? attributes[20]:0.0
		@smooth=attributes[21] ? attributes[21]:"Off"
		
	end

	def invert
	
		b=@b.to_f
		c=@c.to_f
		@c=@b.to_f+c.to_f
		@b=-1.0*b.to_f

		return self
	end
		
	def procedural?

		case type
		when "Fractal", "Shader", "Noise"
			return true
		else
			return false
		end
	end

	def shader_code
		case type
		when "Fractal"
			return "def eval(vec3 pos) vec3:vec3(add(mul(fbm(mul(getTexCoords(0),5.0),10),1.0),0.0))"
		when "Noise"
			return "def eval(vec3 pos) vec3:vec3(add(mul(noise(mul(getTexCoords(0),5.0)),1.0),0.0))"
		else
			return @shader_code
		end
	end

	def preview_uvset
		@@preview_uvset
	end

	def filename
		return File.basename(@path)
	end

	def valid?
		case @type
		when "SketchUp"
			return true
		when "External"
			path = @path.to_s
			return FileTest.exist?(path)
		when "Shader"
			return true
		when "Constant"
			return true
		end
	end

	def valid_reference?

		case @type
		when "SketchUp"
			return true
		when "External"
			path=@path.to_s
			if ((FileTest.exist?(path)) or (path==""))
				return true
			else
				return false
			end
		when "Shader"
			return true
		when "Constant"
			return true
		end
	end


	def uv_set_index()
		return 0 if @uv_set=="Default"
		return @uv_set.to_i
	end

	def to_s
		return [type, uv_set, a, b, c, exponent, shader_code, path, spectrum_type, bb_temperature, bb_gain, rgb_r, rgb_g, rgb_b, rgb_gain, rgb_gamma, reg_start, reg_end, reg_num_values, reg_values, uni_value, smooth].join("|")
	end

	def smooth?
		return smooth == "On"
	end
	
	
	######Defaults
	def default_sss_map
		@type="Constant"
		@spectrum_type="Uniform"
		@uni_value=0.0
		@rgb_gamma=1.0
		return self
	end
	###
	def default_g_spectrum
		@type="Constant"
		@spectrum_type="Uniform"
		@uni_value=1.0
		@rgb_gamma=1.0
		return self
	end
	####
	def default_base_emission
		@type="Constant"
		@spectrum_type="Blackbody"
		@bb_temperature=5000
		@bb_gain=1.0
		return self
	end

	def default_bump
		@type="External"
		@shader_code=self.default_wi_shader
		@b=0.01
		@exponent=2.2
		return self
	end

	def default_normal
		@type="External"
		@shader_code=self.default_wd_shader
		@b=1.0
		@exponent=1.0
		return self
	end

	def default_albedo
		@type="SketchUp"
		@b=1.0
		@exponent=2.2
		@shader_code=self.default_wd_shader
		return self
	end

	def default_exponent
		@type="Constant"
		@b=100
		@exponent=2.2
		@shader_code=self.default_wi_shader
		return self
	end

	def default_clip
		@type="External"
		@b=1.0
		@exponent=2.2
		@shader_code=self.default_wi_shader
		return self
	end

	def default_displacement
		@type="External"
		@b=0.1
		@exponent=2.2
		@shader_code=self.default_wi_shader
		return self
	end

	def default_emission
		@type="External"
		@b=1.0
		@exponent=2.2
		@shader_code=self.default_wd_shader
		return self
	end

	def default_transmittance
		@type="Constant"
		@spectrum_type="Uniform"
		@uni_value=1.0
		@rgb_gamma=1.0
		@shader_code=self.default_wd_shader
		return self
	end

	def default_sigma
		@type="Constant"
		@b=0.0
		@exponent=2.2
		@shader_code=self.default_wi_shader
		return self
	end

	def default_r_f
		@type="Constant"
		@b=0.5
		@exponent=2.2
		@shader_code=self.default_wi_shader
		return self
	end

	def default_roughness
		type="Constant"
		@b=0.3
		@exponent=2.2
		@shader_code=self.default_wi_shader
		return self
	end

	def default_fresnel
		type="Constant"
		@b=0.75
		@exponent=2.2
		@shader_code=self.default_wi_shader
		return self
	end

	def default_thickness
		type="Constant"
		@b=0.001
		@exponent=2.2
		@shader_code=self.default_wi_shader
		return self
	end

	def default_specular
		@type="SketchUp"
		@b=1.0
		@exponent=2.2
		@shader_code=self.default_wd_shader
		return self
	end

	def default_abs_layer_trans
		@type="External"
		@b=1.0
		@exponent=2.2
		@shader_code=self.default_wd_shader
		return self
	end

	def default_absorption
		@type="SketchUp"
		@shader_code=self.default_wd_shader
		@rgb_gamma=1.0
		return self
	end

	def default_blend
		@type="SketchUp"
		@b=1.0
		@exponent=2.2
		@shader_code=self.default_wi_shader
		return self
	end

	####
	def default_wi_shader  #default wavelength-indepdent shader

		return "def eval() real : fbm(posOS()*4.0, 2)*0.02
			#This is a fractal noise shader.
			#4.0=Position scale
			#2=fbm octave
			#0.02=multiplier (eg. height for displacement or bump)"
	end

	def default_wd_shader
		return "def eval(vec3 pos) vec3:vec3(0.4, 0.4, 0.4) # Return a grey colour"
	end
	
	###returns the color of the spectrum for setting a swatch color
	def color

		case @spectrum_type
		when "RGB"
			red=@rgb_r.to_f.abs
			green=@rgb_g.to_f.abs
			blue=@rgb_b.to_f.abs
			color_vec=[red,green,blue]# .normalize!
			color_vec=color_vec.collect {|e| (e*255).to_i}
			return color_vec
		when "Tabulated"
			return self.tabulated_to_rgb(@reg_values.split(" "),@reg_start.to_i,@reg_end.to_i)
		when "Blackbody"
			color=self.blackbody_to_rgb(@bb_temperature.to_i)
			return color.collect {|c| (c*255).to_i}
		when "Uniform"
			color=[]
			3.times {|i| color[i]=(@uni_value.to_f*255).to_i.abs}
			return color
		end
	end

end #end of class IndigoTexture
