#SkIndigo Ruby code is copyright Dale Martens 2009, all rights reserved.
class SkindigoObjectEditor < Sketchup::SelectionObserver

	include SkIndigoUVS

	def initialize
		@model=Sketchup.active_model
		@dict="indigo07_settings"
		@on_mac=SkIndigo.on_mac?
		@selection=@model.selection
		@irs=IndigoRenderSettings.new
		
		@dlg=UI::WebDialog.new("SkIndigo Mesh Settings", false, "SkIndigoObjectEditor",278, 390, 200,200,true)
		plugins=Sketchup.find_support_file("Plugins\/skindigo")
		fn=File.join(plugins,"ObjectEditor.html")
		@dlg.set_file fn
		color = @dlg.get_default_dialog_color 
		@dlg.set_background_color(color)
		@dlg.set_on_close {}
		
		@dlg.add_action_callback("load_uv_texture") {|d,p|
			textures=@irs.get_model_textures
			current_tex=d.get_element_value("TexturePath")
			new_tex=textures.find {|path| File.basename(path)==current_tex}
			ents=Sketchup.active_model.active_entities
			uv_set=@dlg.get_element_value("uvset").to_i
			if new_tex
				faces=ents.find_all {|e| e.class==Sketchup::Face}
				Sketchup.active_model.start_operation("Load UV Texture")
				store_uvs(faces,uv_set)
				uv_mat=get_uv_mat()
				uv_mat.texture=new_tex
				@imesh.uv_texture=true
				faces.each {|f|
					mat=f.material
					unless mat==uv_mat
						f.back_material=mat
						f.material=uv_mat
					end
				}
				position_map(faces,uv_set)
				Sketchup.active_model.commit_operation
			elsif current_tex=="SketchUp"
				faces=ents.find_all {|e| e.class==Sketchup::Face}
				Sketchup.active_model.start_operation("Load UV Texture")
				store_uvs(faces,uv_set)
				faces.each {|f|
					f.material=f.back_material
					f.back_material=nil
				}
				position_map(faces,uv_set)
				@imesh.uv_texture=false
				Sketchup.active_model.commit_operation
			elsif current_tex=="Add New.."
				filename = SkIndigo.openpanel_sketchup_supported_image("Load New Texture")
				return if not filename
				if filename[-3..-1].downcase=="gif"
					UI.messagebox("GIF textures are not supported.")
				else
					faces=ents.find_all {|e| e.class==Sketchup::Face}
					Sketchup.active_model.start_operation("Load UV Texture")
					store_uvs(faces,uv_set)
					uv_mat=get_uv_mat()
					uv_mat.texture=filename
					@irs.store_texture(filename)
					@imesh.uv_texture=true
					faces.each {|f|
						mat=f.material
						unless mat==uv_mat
							f.back_material=mat
							f.material=uv_mat
						end
					}
					position_map(faces,uv_set)
					refresh_dialog()
					Sketchup.active_model.commit_operation
				end
			end
		}
		
		@dlg.add_action_callback("load_uv_set") {|d,p|
			SkIndigo.start_operation("Change UV Set")
			t=Time.now
			uv_set=@imesh.uv_set.to_i  #get the previous uv_set layer
			new_uv_set=@dlg.get_element_value("uvset").to_i
			#store the current uvs to this set
			ents=@imesh.definition.entities
			faces=ents.find_all {|e| e.class==Sketchup::Face}
			if faces.length>0
				@imesh.add_uvset(new_uv_set)	
				store_uvs(faces,uv_set)
				#UI.messagebox("stored_uvs")
				position_map(faces,new_uv_set)
				#UI.messagebox("loaded uv set")
			end
			#new_uv_set="Default" if new_uv_set==0
			@imesh.uv_set=new_uv_set
			#p "Changed uv set in #{Time.now-t} seconds."
			SkIndigo.commit_operation
			#defn=@imesh.definition
			#defn.set_attribute(@dict,'uv_set',uv_set)
			
			#stored_set=defn.get_attribute('uvsets',uv_set)
			#if stored_set==true
			#	UI.messagebox("UV set #{uv_set} loaded.")
			#else
			#	UI.messagebox("No UV data to load for UV set #{uv_set}.")
			#end
		}
		
		#############Primary Event Handler Callback Function for Material Editor##################
		@dlg.add_action_callback("ValueChanged") {|d,p|
			pair=p.split("=")
			id=pair[0]           
			value=pair[1]
			case id
				when "maxSubdiv"
					@imesh.max_subdiv=value
				when "pixelThreshold"
					@imesh.pixel_threshold=value
				when "curveThreshold"
					@imesh.curvature_threshold=value
				when "viewDependentCheckbox"
					@imesh.view_dependent_subdivision=true if value=="true"
					@imesh.view_dependent_subdivision=false if value=="false"
				when "subSmoothingCheckbox"
					@imesh.subdivision_smoothing=true if value=="true"
					@imesh.subdivision_smoothing=false if value=="false"
				when "externalMeshBrowse"
					if SkIndigo.sketchup_is_2014_or_later # Multiple file type wildcards were broken before SketchUp 2014: http://www.sketchup.com/intl/en/developer/docs/ourdoc/ui?csw=1#openpanel
						path = UI.openpanel("Select OBJ, IGMESH, or PLY", "", "Mesh file (*.obj;*.igmesh;*.ply)|*.obj;*.igmesh;*.ply||")
					else
						path = UI.openpanel("Select OBJ, IGMESH, or PLY", "", "Mesh file (.obj;.igmesh;.ply)")
					end
			
					if path
						#UI.messagebox("When using an external mesh, you must ensure that your SketchUp model contains materials with identical names to those within the external mesh file.  Naming conflicts may create unexpected results.  Beware of duplicate material names when using SketchUp (or another Ruby script) to import the external mesh file in order to import the mesh materials.")
						@imesh.external_mesh_path=path.gsub(/\\/,"\/")
						@imesh.use_external=true	
						refresh_dialog()					
					end
				when "externalMeshPath"
					@imesh.external_mesh_path=value
				when "externalMeshCheckbox"
					@imesh.use_external=true if value=="true"
					@imesh.use_external=true if value=="false"
				when "maskName"
					@imesh.mask_name=value
			end
		}
		
		@dlg.add_action_callback("RefreshValues") {|d,p| 
				refresh_dialog()	
		}
		
	end #init

	def dialog
		return @dlg
	end

	def refreshTexturesList()
		cmd="clearTexturesList()"
		@dlg.execute_script(cmd)
		add_map_option("SketchUp")
		getModelTextures()
		add_map_option("Add New..")
	end

	def onSelectionCleared(selection)
		#p "clear selection"
		#@imesh=nil
		# refresh_dialog()
	end
	 

	def onSelectionBulkChange(selection)
		# p ("onSelectionBulkChange:")
		#model=Sketchup.active_model
		# model.start_operation("invalidate_mesh",false,false)
		#path=model.active_path
		#if path!=nil
		#	e=path.last
		#	if e.class==Sketchup::Group
		#		definition=e.entities.parent
		#	else
		#		definition=e.definition
		#	end
		#	definition.set_attribute('indigo07_settings','valid_guid',false)
		#end
		#model.commit_operation
		#p Sketchup.active_model.active_path
		if self.instance_selected?
			#p "instance selected"
			e=@selection.first
			e.add_observer(SkIndigoInstanceObserver.new)
			#definition=Sketchup.active_model.active_entities.parent
			#if e.class==Sketchup::Group
			#	definition=e.entities.parent
			#else
			#	definition=e.definition
			#end
			#@ii=IndigoInstance.new(e)
			#@imesh=IndigoMesh.new(definition)
		else
			#@imesh=nil
		end
		#refresh_dialog()
	end

	###
	def refresh_dialog()
		definition = Sketchup.active_model.active_entities.parent
		@imesh = IndigoMesh.new(definition)
		refreshTexturesList()
		loadValuesFromSU()
	end

	###
	def hide_ui()
		cmd="$('#mainUI').hide()"
		@dlg.execute_script(cmd)
	end

	def show_ui()
		cmd="$('#mainUI').show()"
		@dlg.execute_script(cmd)
	end
	
	######
	def show(defn=nil)
		if defn!=nil
			@imesh=IndigoMesh.new(defn)
		end

		if @dlg.visible?
			self.refresh_dialog()
		else
			if @on_mac
				self.refresh_dialog()
				@dlg.show_modal{}
			else
				@dlg.show{}
			end
		end
	end #show

	####
	def instance_selected?

		#return true unless Sketchup.active_model.active_entities.parent.class==Sketchup::Model

		return false unless @selection.single_object?
		e = @selection.first
		case e
		when Sketchup::ComponentInstance then
			return true
		when Sketchup::Group then
			return true	
		else
			return false
		end
	end
	
	#####
	def loadValuesFromSU()

		if @imesh==nil
			title="No mesh selected"
			#set the heading of the dialog
			cmd="$('#meshName').html('#{title}')"
			@dlg.execute_script(cmd)
			self.hide_ui()
			return
		end

		self.show_ui()

		setValue("maxSubdiv",@imesh.max_subdiv)
		setValue("pixelThreshold",@imesh.pixel_threshold)
		setValue("curveThreshold",@imesh.curvature_threshold)
		setValue("externalMeshPath",@imesh.external_mesh_path)
		setValue("maskName",@imesh.mask_name)
		setValue("uvset",@imesh.uv_set)
		tex_path="SketchUp"
		if @imesh.uv_texture?
			uv_mat=get_uv_mat()
			if uv_mat.texture!=nil
				tex_path=uv_mat.texture.filename.gsub(/\\/,"\/")
				tex_path=File.basename(tex_path)
			end
		end
		#p tex_path
		setValue("TexturePath",tex_path) 

		if  @imesh.view_dependent_subdivision? then setCheckbox("viewDependentCheckbox","true")
			else setCheckbox("viewDependentCheckbox","false")
		end
		if  @imesh.subdivision_smoothing? then setCheckbox("subSmoothingCheckbox","true")
			else setCheckbox("subSmoothingCheckbox","false")
		end
		if  @imesh.use_external? then setCheckbox("externalMeshCheckbox","true")
			else setCheckbox("externalMeshCheckbox","false")
		end

		#count all of the faces in the definition
		entities=@imesh.definition.entities
		faces=entities.find_all {|e| e.class==Sketchup::Face}
		num_faces=faces.length
		title="Active Mesh: #{@imesh.name} (#{num_faces} faces)"
		#set the heading of the dialog
		cmd="$('#meshName').html('#{title}')"
		@dlg.execute_script(cmd)	
	end #load values

	def setValue(id,value)
		new_value=value.to_s
		cmd="document.getElementById('#{id}').value=\"#{new_value}\""
		@dlg.execute_script(cmd)
	end

	def setCheckbox(id,checked)
		cmd="document.getElementById('#{id}').checked=#{checked}"
		@dlg.execute_script(cmd)
	end

	def getModelTextures()

		@model_textures=@irs.get_model_textures
		@model_textures.each {|path|
			add_map_option(File.basename(path))
		}
	end

	def add_map_option(path)
		path=path.gsub(/\\/,"\/")
		cmd="createMapOption(\"TexturePath\",\"#{path}\",\"#{path}\")"
		@dlg.execute_script(cmd)
	end

end #class


#################
class SkIndigoInstanceObserver < Sketchup::InstanceObserver
	def onOpen(instance)
		$skindigo_object_editor.refresh_dialog() if $skindigo_object_editor
	end

	def onClose(instance)
		$skindigo_object_editor.refresh_dialog() if $skindigo_object_editor
		instance.remove_observer(self)
	end
end
