#SkIndigo Ruby code is copyright Dale Martens 2009, all rights reserved.


##########################new module for UV management
module SkIndigoUVS

def store_uvs(ents,uvset)

uvset=0 if uvset==nil

face=0
case uvset
	when 0
		key="uv0"
	when 1
		key="uv1"
	when 2
		key="uv2"
	when 3
		key="uv3"
end



for e in ents
	
	if (e.valid?) and (e.class==Sketchup::Face)
		mat=e.material
		next if mat==nil
		next if mat.texture==nil
		polymesh=e.mesh 5
		uvs=polymesh.uvs 5
		for i in (0..uvs.length-1)
			uvs[i]=uvs[i].to_a  #converts the Point 3D objects to arrays so they won't be transformed
			#uv_w=uvs[i].z
			#uvs[i].x=uvs[i].x/uv_w
			#uvs[i].y=uvs[i].y/uv_w
			#uvs[i].z=uvs[i].z/uv_w
		end
		e.set_attribute 'uvs',key,uvs
	
	face=face+1
	Sketchup.set_status_text("Stored uvs for #{face} faces.")
	end
		
end

#UI.messagebox "UVs saved for #{face} faces."

if face>0
	defn=Sketchup.active_model.active_entities.parent
	defn.set_attribute('uvsets',uvset,true)
end



end #function

##############returns the uv mat helper material
def get_uv_mat()

name="SkIndigo UV Helper"

mats=Sketchup.active_model.materials
uv_mat=mats[name]
if uv_mat==nil
	uv_mat=mats.add(name)
end

return uv_mat

end

##
def restore_material()

#fix so that this will only apply to faces with UV Helper

ents=Sketchup.active_model.active_entities
defn=ents.parent
uv_set=defn.get_attribute("indigo07_materials",'uv_set',0)

faces=ents.find_all {|e| e.class==Sketchup::Face}
store_uvs(faces,uv_set)
faces.each {|f|
	f.material=f.back_material
	f.back_material=nil
	if f.material.texture!=nil
		position_map([f],uv_set)
	end
}


end

################
def switch_mat(ents,old_mat,new_mat,uv_set)

switched=[]
#mat_name=old_mat.name
#ents.each {|e|
	#if e.valid? and e.material==old_mat
		#switched.push(e)
	#end
#}

#switched.each {|e|
ents.each {|e|
	#if e.material!=nil
	#	mat_name=e.material.name
	#else
	#	mat_name=nil
	#end
	e.material=new_mat
	#e.set_attribute('indigo_uv','mat_name',mat_name)  #store the current material name as an attribute
	e.back_material=old_mat  #save the old material by applying it the back face
}


end  #end switch mat

###################position a texture on a material from a stored UV set
def position_map(entities,set)

face=0
failed=0
return if set=="SU"  
key="uv"+set.to_s

for e in entities
	next if e.material==nil
	if (e.valid?) and (e.class==Sketchup::Face)
		
		uvs=e.get_attribute 'uvs',key    #gets the stored array of uv coordinates for this face for the given uv set
		#UI.messagebox(uvs)
		if uvs
			pos=[]
			polymesh=e.mesh 5   #get a polygon mesh representation of the face
			polygons=polymesh.polygons  
			poly_index=0
			begin
				for p in polygons[poly_index]
					point=polymesh.point_at(p.abs)
					if point
						pos.push(point) 
						pos.push(uvs[p.abs-1])
					end
				end

				e.position_material e.material, pos, true
				face=face+1
				Sketchup.set_status_text("Loaded uvs for #{face} faces")
			rescue  #this is required because SketchUp sometimes fails to position the texture properly
				poly_index=poly_index+1
				if polygons[poly_index]
					pos=[]
					retry  #try to postion texture again using next polygon in the face
				else
					failed=failed+1
				end
			end
	
			
		end
	end
end

if failed>0
	stext="Unable to load
	UVs for #{failed} faces.  Loaded UVs for #{face} faces."
else
	stext="Loaded UVs for #{face} faces."
end

#UI.messagebox (stext)


end  #end position map

##############
def save_mat_texture(mat,path)   #mat is a SU material, path is the folder

#p "saving #{mat.display_name} texture at path #{path}"
Sketchup.active_model.start_operation("save_mat")
tw=Sketchup.create_texture_writer
entities = Sketchup.active_model.active_entities
helper_group=entities.add_group
if (mat.respond_to?(:texture) and mat.texture !=nil)
	helper_group.material=mat
	tw.load(helper_group)
	status = tw.write(helper_group, path)
end	
#entities.erase_entities helper_group
Sketchup.active_model.abort_operation
tw=nil
if status==0   #write successful so apply the image to the dialog
	return path
else
	return nil
end

end  

###########
def restore_old_mat(ents)

#p "restoring old mat"
materials=Sketchup.active_model.materials
status=true
ents.each {|e|
	old_mat_name=e.get_attribute('indigo_uv','mat_name') #retrieve stored material name
	if old_mat_name!=nil
		old_mat=materials[old_mat_name]
		if old_mat==nil
			status=false
		else
			#p old_mat.name
			#p e.material.name
			e.material=old_mat
			if old_mat.texture!=nil
				position_map([e],0)   #restore the old uv set
			end
		end
	else
		#e.material=nil  #set to default material
	end
}

return status

end

################
def purge_uvs(ents)

result=UI.messagebox "Would you like to purge all stored UV sets for the currently active entities?",MB_YESNO
return if result!=6

Sketchup.active_model.start_operation "Purge UV sets"

faces=0
ents=Sketchup.active_model.active_entities
for e in ents
	if (e.valid?) and (e.class==Sketchup::Face)
		e.delete_attribute 'uvs'
		faces=faces+1
	end
end  


UI.messagebox "Purged UV sets for #{faces} faces."

defn=Sketchup.active_model.active_entities.parent
defn.delete_attribute('uvsets')

Sketchup.active_model.commit_operation

end

end  #end UV module